from hachoir.parser import createParser
from hachoir.editor import createEditor
from hachoir.field import writeIntoFile

parser = createParser("file.gz")
with parser:
    editor = createEditor(parser)
    del editor["filename"]
    editor["has_filename"].value = False
    writeIntoFile(editor, "noname.gz")



#!/usr/bin/env python3
"""
Extract audio from a FLV movie

Author: Victor Stinner
Creation date: 2006-11-06
"""
from hachoir.parser import createParser
from hachoir.stream import FileOutputStream
from hachoir.parser.video.flv import AUDIO_CODEC_MP3
from sys import stderr, exit, argv


def main():
    if len(argv) != 2:
        print("usage: %s video.flv" % argv[0], file=stderr)
        exit(1)

    # Open input video
    inputname = argv[1]
    parser = createParser(inputname)
    if parser["audio[0]/codec"].value != AUDIO_CODEC_MP3:
        print("Unknown audio codec: %s" %
              parser["audio[0]/codec"].display, file=stderr)

    # Extract audio
    print("Extractor audio from: %s" % inputname)
    outputname = inputname + ".mp3"
    output = FileOutputStream(outputname)
    for chunk in parser.array("audio"):
        data = chunk["music_data"]
        output.copyBitsFrom(
            data.parent.stream, data.absolute_address, data.size, data.parent.endian)
    print("Write audio into: %s" % outputname)


main()



#!/usr/bin/env python3
from hachoir.test import setup_tests
import hashlib
import os.path
import subprocess
import sys
import unittest

DATADIR = os.path.join(os.path.dirname(__file__), "files")
KDE_CLICK = os.path.join(DATADIR, 'kde_click.wav')
PROGRAM = os.path.join(os.path.dirname(__file__), "..", "hachoir-strip")


def checksum(filename):
    hash = hashlib.sha1()
    with open(filename, 'rb') as fp:
        chunk = fp.read(4096)
        hash.update(chunk)
    return hash.hexdigest()


class TestStripCommandLine(unittest.TestCase):

    def test_strip_all(self):
        self.assertEqual(checksum(KDE_CLICK),
                         'dcafdef2048985aa925df5f86053bda5a87eb64b')

        newname = KDE_CLICK + ".new"
        if os.path.exists(newname):
            os.unlink(newname)

        args = [sys.executable, PROGRAM, KDE_CLICK]
        proc = subprocess.Popen(args,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.STDOUT)
        stdout, _ = proc.communicate()
        stdout = stdout.decode('ascii', 'replace')
        self.assertIn('Save new file', stdout)

        self.assertEqual(checksum(newname),
                         '6456990d3931292a1c96c6e8f035e983cd84d477')

        os.unlink(newname)


if __name__ == "__main__":
    setup_tests()
    unittest.main()



from hachoir.parser import createParser
from hachoir.editor import createEditor
from hachoir.field import writeIntoFile
from hachoir.editor import EditableInteger, EditableBytes

parser = createParser("file.gz")
with parser:
    editor = createEditor(parser)
    extra = "abcd"
    editor["has_extra"].value = True
    editor.insertAfter("os",
                       EditableInteger(editor, "extra_length", False,
                                       16, len(extra)),
                       EditableBytes(editor, "extra", extra))
    writeIntoFile(editor, "file_extra.gz")



#!/usr/bin/env python3

from hachoir.editor import (createEditor as hachoirCreateEditor,
                            NewFieldSet, EditableInteger, EditableString, EditableBytes)
from hachoir.stream import FileOutputStream
from hachoir.parser import createParser
from hachoir.parser.image import PngFile
from hachoir.parser.audio import MpegAudioFile
from sys import argv, stdin, stdout, stderr, exit
import zlib


class InjecterError(Exception):
    pass


class Injecter:

    def __init__(self, editor):
        self.editor = editor

    def getMaxSize(self):
        "None: no limit"
        raise NotImplementedError()

    def read(self):
        raise NotImplementedError()

    def write(self, data):
        raise NotImplementedError()

    def saveInto(self, filename):
        output = FileOutputStream(filename)
        self.editor.writeInto(output)


def computeCRC32(data):
    "Compute CRC-32 of data string. Result is a positive integer."
    crc = zlib.crc32(data)
    if 0 <= crc:
        return crc
    else:
        return 1 << 32


class PngInjecter(Injecter):
    MAGIC = "HACHOIR"

    def getMaxSize(self):
        return None

    def read(self):
        for field in self.editor:
            if field.name.startswith("text[") \
                    and field["keyword"].value == self.MAGIC:
                return field["text"].value
        return None

    def write(self, data):
        tag = "tEXt"
        data = "%s\0%s" % (self.MAGIC, data)
        size = len(data)
        crc = computeCRC32(tag + data)
        chunk = NewFieldSet(self.editor, "inject[]")
        chunk.insert(EditableInteger(chunk, "size", False, 32, size))
        chunk.insert(EditableBytes(chunk, "tag", tag))
        chunk.insert(EditableBytes(chunk, "content", data))
        chunk.insert(EditableInteger(chunk, "crc32", False, 32, crc))
        self.editor.insertBefore("end", chunk)


class MpegAudioInjecter(Injecter):
    MAX_PACKET_SIZE = 2048  # bytes between each frame

    def __init__(self, editor, packet_size=None):
        Injecter.__init__(self, editor)
        self.frames = editor["frames"]
        if packet_size:
            # Limit packet size to 1..MAX_PACKET_SIZE bytes
            self.packet_size = max(min(self.MAX_PACKET_SIZE, packet_size), 1)
        else:
            self.packet_size = self.MAX_PACKET_SIZE

    def getMaxSize(self):
        return len(self.frames) * self.packet_size * 8

    def read(self):
        data = []
        for field in self.frames:
            if field.name.startswith("padding["):
                data.append(field.value)
        if data:
            return b"".join(data)
        else:
            return None

    def write(self, data):
        count = 30
        self.packet_size = 3
        data = b"\0" * (self.packet_size * count - 1)
        print("Packet size: %s" % self.packet_size)
        print("Check input message")
        if b"\xff" in data:
            raise InjecterError(
                "Sorry, MPEG audio injecter disallows 0xFF byte")

#        print "Check message size"
#        maxbytes = self.getMaxSize()
#        if maxbytes < len(data)*8:
#            raise InjecterError("Message is too big (max: %s, want: %s)" % \
#                (maxbytes, len(data)))

        print("Inject message")
        field_index = 0
        index = 0
        output = self.frames
        while index < len(data):
            padding = data[index:index + self.packet_size]
            print(index, index + self.packet_size, type(padding))
            name = "frame[%u]" % field_index
            print("Insert %s before %s" % (len(padding), name))
            output.insertAfter(name, EditableString(
                output, "padding[]", "fixed", padding))
            index += self.packet_size
            field_index += 2


def createEditor(filename):
    parser = createParser(filename)
    return hachoirCreateEditor(parser)


injecter_cls = {
    PngFile: PngInjecter,
    MpegAudioFile: MpegAudioInjecter,
}


def main():
    if len(argv) != 2:
        print("usage: %s music.mp3" % argv[0], file=stderr)
        exit(1)

    filename = str(argv[1])
    editor = createEditor(filename)
#    injecter = injecter_cls[editor.input.__class__]
    injecter = MpegAudioInjecter(editor, packet_size=16)

    if False:
        data = injecter.read()
        if data:
            stdout.write(data)
            exit(0)
        else:
            print("No data", file=stderr)
            exit(1)
    else:
        out_filename = filename + ".msg"
        print("Write your message and valid with CTRL+D:")
        stdout.flush()
        data = stdin.read()
        data = data.encode('utf-8')

        print("Hide message")
        injecter.write(data)

        print("Write ouput into: %s" % out_filename)
        injecter.saveInto(out_filename)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
Proof of concept of Hachoir user interface using FUSE
"""

import errno
# import os
import stat
from hachoir.log import log
from hachoir.field import MissingField
from hachoir.tools import makePrintable
from hachoir.stream import FileOutputStream
from hachoir.editor import createEditor
from hachoir.parser import createParser

# some spaghetti to make it usable without fuse-py being installed
for i in True, False:
    try:
        import fuse
        from fuse import Fuse
    except ImportError:
        if i:
            try:
                import _find_fuse_parts   # noqa
            except ImportError:
                pass
        else:
            raise

if not hasattr(fuse, '__version__'):
    raise RuntimeError(
        "your fuse-py doesn't know of fuse.__version__, probably it's too old.")

# This setting is optional, but it ensures that this class will keep
# working after a future API revision
fuse.fuse_python_api = (0, 2)


class MyStat(fuse.Stat):

    def __init__(self):
        self.st_mode = 0
        self.st_ino = 0
        self.st_dev = 0
        self.st_nlink = 0
        self.st_uid = 0
        self.st_gid = 0
        self.st_size = 0
        self.st_atime = 0
        self.st_mtime = 0
        self.st_ctime = 0


class HelloFS(Fuse):

    def __init__(self, input_filename, **kw):
        Fuse.__init__(self, **kw)
        log.setFilename("/home/haypo/fuse_log")
        self.hachoir = createParser(input_filename)
        if True:
            self.hachoir = createEditor(self.hachoir)
            self.readonly = False
        else:
            self.readonly = True
        self.fs_charset = "ASCII"

    def getField(self, path):
        try:
            field = self.hachoir
            try:
                for name in path.split("/")[1:]:
                    if not name:
                        break
                    name = name.split("-", 1)
                    if len(name) != 2:
                        return None
                    field = field[name[1]]
                return field
            except MissingField:
                return None
        except Exception as xx:
            log.info("Exception: %s" % str(xx))
            raise

    def fieldValue(self, field):
        return makePrintable(field.display, "ISO-8859-1") + "\n"

    def getattr(self, path):
        st = MyStat()
        if path == '/':
            st.st_mode = stat.S_IFDIR | 0o755
            st.st_nlink = 2
            return st
        if path == "/.command":
            st.st_mode = stat.S_IFDIR | 0o755
            return st
        if path.startswith("/.command/"):
            name = path.split("/", 3)[2]
            if name in ("writeInto",):
                st.st_mode = stat.S_IFREG | 0o444
                return st
            return -errno.ENOENT

        # Get field
        field = self.getField(path)
        if not field:
            return -errno.ENOENT

        # Set size and mode
        if field.is_field_set:
            st.st_mode = stat.S_IFDIR | 0o755
        else:
            st.st_mode = stat.S_IFREG | 0o444
        st.st_nlink = 1
        if field.hasValue():
            st.st_size = len(self.fieldValue(field))
        else:
            st.st_size = 0
        return st

    def unlink(self, path, *args):
        log.info("unlink(%s)" % path)
        field = self.getField(path)
        log.info("del %s" % field.name)
        if not field:
            return -errno.ENOENT
        if self.readonly:
            return -errno.EACCES
        log.info("del %s" % field.name)
        try:
            del field.parent[field.name]
        except Exception as err:
            log.info("del ERROR %s" % err)
        return 0

    def readCommandDir(self):
        yield fuse.Direntry('writeInto')

    def readdir(self, path, offset):
        log.info("readdir(%s)" % path)
        yield fuse.Direntry('.')
        yield fuse.Direntry('..')
        if path == "/.command":
            for entry in self.readCommandDir():
                yield entry
            return

        # Get field
        fieldset = self.getField(path)
#        if not fieldset:
#            return -errno.ENOENT

        if path == "/":
            entry = fuse.Direntry(".command")
            entry.type = stat.S_IFREG
            yield entry

        # Format file name
        count = len(fieldset)
        if count % 10:
            count += 10 - (count % 10)
        format = "%%0%ud-%%s" % (count // 10)

        # Create entries
        for index, field in enumerate(fieldset):
            name = format % (1 + index, field.name)
            entry = fuse.Direntry(name)
            if field.is_field_set:
                entry.type = stat.S_IFDIR
            else:
                entry.type = stat.S_IFREG
            yield entry
        log.info("readdir(%s) done" % path)

    def open(self, path, flags):
        log.info("open(%s)" % path)
#        if ...:
#            return -errno.ENOENT
#        accmode = os.O_RDONLY | os.O_WRONLY | os.O_RDWR
#        if (flags & accmode) != os.O_RDONLY:
#            return -errno.EACCES

    def write(self, path, data, offset):
        if path == "/.command/writeInto":
            if self.readonly:
                return -errno.EACCES
            try:
                data = data.strip(" \t\r\n\0")
                filename = str(data, self.fs_charset)
            except UnicodeDecodeError:
                log.info("writeInto(): unicode error!")
                return 0
            log.info("writeInto(%s)" % filename)
            stream = FileOutputStream(filename)
            self.hachoir.writeInto(stream)
        return len(data)

    def read(self, path, size, offset):
        try:
            log.info("read(%s, %s, %s)" % (path, size, offset))
            field = self.getField(path)
            if not field:
                log.info("=> ENOENT")
                return -errno.ENOENT
            if not field.hasValue():
                return ''
        except Exception as xx:
            log.info("ERR: %s" % xx)
            raise
        data = self.fieldValue(field)
        slen = len(data)
        if offset >= slen:
            return ''
        if offset + size > slen:
            size = slen - offset
        data = data[offset:offset + size]
        log.info("=> %s" % repr(data))
        return data

    def stop(self):
        log.info("stop()")

    def truncate(self, *args):
        log.info("truncate(): TODO!")


def main():
    usage = """
Userspace hello example

""" + Fuse.fusage
    server = HelloFS('/home/haypo/testcase/KDE_Click.wav',
                     version="%prog " + fuse.__version__,
                     usage=usage,
                     dash_s_do='setsingle')

    server.parse(errex=1)
    server.main()
    server.stop()


if __name__ == '__main__':
    main()



from os import path
from os.path import basename
from random import randint
from tools import getFilesize, generateUniqueID
from hachoir.stream import InputIOStream, InputStreamError
from hachoir.metadata import extractMetadata
from hachoir.parser import guessParser
from io import StringIO
from array import array
from mangle import mangle
from time import time

# Truncate: minimum/maximum file size (in bytes)
MIN_SIZE = 1
MAX_SIZE = 1024 * 1024

# Limit each test to 10 secondes
MAX_DURATION = 10.0

# Number of mangle operation depending of current size
# '0.30' means: 30% of current size in byte
MANGLE_PERCENT = 0.01
MANGLE_PERCENT_INCR = 0
MIN_MANGLE_PERCENT = 0.10

# Limit fuzzing to 40 tests
MAX_NB_EXTRACT = 40

# 1 times on 20 tries
TRUNCATE_RATE = 20


class UndoMangle:

    def __init__(self, fuzz):
        self.data = fuzz.data.tostring()
        self.orig = fuzz.is_original

    def __call__(self, fuzz):
        fuzz.data = array('B', self.data)
        fuzz.is_original = self.orig


class UndoTruncate:

    def __init__(self, fuzz):
        self.data = fuzz.data
        self.orig = fuzz.is_original

    def __call__(self, fuzz):
        fuzz.data = self.data
        fuzz.is_original = self.orig


class FileFuzzer:

    def __init__(self, fuzzer, filename):
        self.fuzzer = fuzzer
        self.verbose = fuzzer.verbose
        self.mangle_percent = MANGLE_PERCENT
        self.file = open(filename, "rb")
        self.nb_undo = 0
        self.filename = filename
        self.size = getFilesize(self.file)
        self.mangle_count = 0
        self.mangle_call = 0
        size = randint(MIN_SIZE, MAX_SIZE)
        data_str = self.file.read(size)
        self.data = array('B', data_str)
        self.undo = None
        self.nb_extract = 0
        if len(self.data) < self.size:
            self.is_original = False
            self.nb_truncate = 1
            self.warning("Truncate to %s bytes" % len(self.data))
        else:
            self.is_original = True
            self.nb_truncate = 0
            self.info("Size: %s bytes" % len(self.data))

    def acceptTruncate(self):
        return MIN_SIZE < len(self.data)

    def sumUp(self):
        self.warning("[SUMUP] Extraction: %s" % self.nb_extract)
        if self.mangle_call:
            self.warning("[SUMUP] Mangle# %s (%s op.)" % (
                self.mangle_call, self.mangle_count))
        if self.nb_truncate:
            percent = len(self.data) * 100.0 / self.size
            self.warning("[SUMUP] Truncate# %s  -- size: %.1f%% of %s" % (
                self.nb_truncate, percent, self.size))

    def warning(self, message):
        print("    %s (%s): %s" %
              (basename(self.filename), self.nb_extract, message))

    def info(self, message):
        if self.verbose:
            self.warning(message)

    def mangle(self):
        # Store last state
        self.undo = UndoMangle(self)

        # Mangle data
        count = mangle(self.data, self.mangle_percent)

        # Update state
        self.mangle_call += 1
        self.mangle_count += count
        self.is_original = False
        self.warning("Mangle #%s (%s op.)" % (self.mangle_call, count))

    def truncate(self):
        assert MIN_SIZE < len(self.data)
        #  Store last state (for undo)
        self.undo = UndoTruncate(self)

        # Truncate
        self.nb_truncate += 1
        new_size = randint(MIN_SIZE, len(self.data) - 1)
        self.warning("Truncate #%s (%s bytes)" % (self.nb_truncate, new_size))
        self.data = self.data[:new_size]
        self.is_original = False

    def tryUndo(self):
        # No operation to undo?
        if not self.undo:
            self.info("Unable to undo")
            return False

        # Undo
        self.nb_undo += 1
        self.info("Undo #%s" % self.nb_undo)
        self.undo(self)
        self.undo = None

        # Update mangle percent
        percent = max(self.mangle_percent -
                      MANGLE_PERCENT_INCR, MIN_MANGLE_PERCENT)
        if self.mangle_percent != percent:
            self.mangle_percent = percent
            self.info("Set mangle percent to: %u%%" %
                      int(self.mangle_percent * 100))
        return True

    def extract(self):
        self.nb_extract += 1
        self.prefix = ""

        data = self.data.tostring()
        stream = InputIOStream(StringIO(data), filename=self.filename)

        # Create parser
        start = time()
        try:
            parser = guessParser(stream)
        except InputStreamError as err:
            parser = None
        if not parser:
            self.info("Unable to create parser: stop")
            return None

        # Extract metadata
        try:
            metadata = extractMetadata(parser, 0.5)
            failure = bool(self.fuzzer.log_error)
        except Exception as err:
            self.info("SERIOUS ERROR: %s" % err)
            self.prefix = "metadata"
            failure = True
        duration = time() - start

        # Timeout?
        if MAX_DURATION < duration:
            self.info("Process is too long: %.1f seconds" % duration)
            failure = True
            self.prefix = "timeout"
        if not failure and (metadata is None or not metadata):
            self.info("Unable to extract metadata")
            return None
#        for line in metadata.exportPlaintext():
#            print(">>> %s" % line)
        return failure

    def keepFile(self, prefix):
        data = self.data.tostring()
        uniq_id = generateUniqueID(data)
        filename = "%s-%s" % (uniq_id, basename(self.filename))
        if prefix:
            filename = "%s-%s" % (prefix, filename)
        filename = path.join(self.fuzzer.error_dir, filename)
        open(filename, "wb").write(data)
        print("=> Store file %s" % filename)



#!/usr/bin/env python3
from hachoir.parser import createParser, guessParser
from sys import stderr, exit, argv


class JpegExtractor:

    def __init__(self):
        self.jpg_index = 1
        self.snd_index = 1
        self.verbose = False

    def storeJPEG(self, content):
        name = "image-%03u.jpg" % self.jpg_index
        print("Write new image: %s" % name)
        with open(name, "wb") as fp:
            fp.write(content)
        self.jpg_index += 1

    def createNewSound(self):
        name = "sound-%03u.mp3" % self.snd_index
        print("Write new sound: %s" % name)
        self.snd_index += 1
        return open(name, "wb")

    def extractFormat2(self, field):
        if "jpeg_header" in field:
            header = field["jpeg_header"]
            if 32 < header.size:
                if self.verbose:
                    print("Use JPEG table: %s" % header.path)
                header = field.root.stream.readBytes(
                    header.absolute_address, (header.size - 16) // 8)
            else:
                header = ""
        else:
            header = None
        content = field["image"].value
        if header:
            content = header + content[2:]
        if self.verbose:
            print("Extract JPEG from %s" % field.path)
        self.storeJPEG(content)

    def extractSound2(self, parser):
        header = None
        output = None
        for field in parser:
            if field.name.startswith("def_sound["):
                header = field
                output = self.createNewSound()
                data = header["music_data"].value
                assert data[:1] == b'\xFF'
                output.write(data)
            elif field.name.startswith("sound_blk") \
                    and "music_data" in field:
                data = field["music_data"].value
                if data:
                    assert data[0] == '\xFF'
                    output.write(data)

    def main(self):
        if len(argv) != 2:
            print("usage: %s document.swf" % argv[0], file=stderr)
            exit(1)

        filename = argv[1]
        parser = createParser(filename)

        if parser["signature"].value == "CWS":
            deflate_swf = parser["compressed_data"].getSubIStream()
            parser = guessParser(deflate_swf)

        if "jpg_table/data" in parser:
            # JPEG pictures with common header
            jpeg_header = parser["jpg_table/data"].value[:-2]
            for field in parser.array("def_bits"):
                jpeg_content = field["image"].value[2:]
                if self.verbose:
                    print("Extract JPEG from %s" % field.path)
                self.storeJPEG(jpeg_header + jpeg_content)

        # JPEG in format 2/3
        for field in parser.array("def_bits_jpeg2"):
            self.extractFormat2(field)
        for field in parser.array("def_bits_jpeg3"):
            self.extractFormat2(field)

        # Extract sound
        # self.extractSound(parser)
        self.extractSound2(parser)

        # Does it extract anything?
        if self.jpg_index == 1:
            print("No JPEG picture found.")
        if self.snd_index == 1:
            print("No sound found.")


JpegExtractor().main()



from hachoir.parser import createParser
from hachoir.metadata import extractMetadata
from sys import argv, stderr, exit

if len(argv) != 2:
    print("usage: %s filename" % argv[0], file=stderr)
    exit(1)
filename = argv[1]
parser = createParser(filename)
if not parser:
    print("Unable to parse file", file=stderr)
    exit(1)

with parser:
    try:
        metadata = extractMetadata(parser)
    except Exception as err:
        print("Metadata extraction error: %s" % err)
        metadata = None
if not metadata:
    print("Unable to extract metadata")
    exit(1)

for line in metadata.exportPlaintext():
    print(line)



#!/usr/bin/env python3
from os import path, getcwd, nice, mkdir
from sys import exit, argv, stderr
from glob import glob
from random import choice as random_choice, randint
from hachoir.core.memory import limitedMemory
from errno import EEXIST
from time import sleep
from hachoir.core.log import log as hachoir_logger, Log
from file_fuzzer import FileFuzzer, MAX_NB_EXTRACT, TRUNCATE_RATE
import re

# Constants
SLEEP_SEC = 0
MEMORY_LIMIT = 5 * 1024 * 1024


class Fuzzer:

    def __init__(self, filedb_dirs, error_dir):
        self.filedb_dirs = filedb_dirs
        self.filedb = []
        self.tmp_file = "/tmp/stress-hachoir"
        self.nb_error = 0
        self.error_dir = error_dir
        self.verbose = False

    def filterError(self, text):
        if "Error during metadata extraction" in text:
            return False
        if text.startswith("Error when creating MIME type"):
            return True
        if text.startswith("Unable to create value: "):
            why = text[24:]
            if why.startswith("Can't get field \""):
                return True
            if why.startswith("invalid literal for int(): "):
                return True
            if why.startswith("timestampUNIX(): value have to be in "):
                return True
            if re.match("^Can't read [0-9]+ bits at ", why):
                return True
            if why.startswith("'decimal' codec can't encode character"):
                return True
            if why.startswith("date newer than year "):
                return True
            if why in (
                "day is out of range for month",
                "year is out of range",
                    "[Float80] floating point overflow"):
                return True
            if re.match("^(second|minute|hour|month) must be in ", why):
                return True
        if re.match("days=[0-9]+; must have magnitude ", text):
            # Error during metadata extraction: days=1143586582; must have
            # magnitude <= 999999999
            return True
        if "floating point overflow" in text:
            return True
        if "field is too large" in text:
            return True
        if "Seek below field set start" in text:
            return True
        if text.startswith("Unable to create directory directory["):
            # [/section_rsrc] Unable to create directory directory[2][0][]: Can't get field "header" from /section_rsrc/directory[2][0][]
            return True
        if "FAT chain: " in text:
            return True
        if text.startswith("EXE resource: depth too high"):
            return True
        if "OLE2: Unable to parse property of type" in text:
            return True
        if "OLE2: Too much sections" in text:
            return True
        if "OLE2: Invalid endian value" in text:
            return True
        if "Seek above field set end" in text:
            return True
        return False

    def newLog(self, level, prefix, text, context):
        if level < Log.LOG_ERROR or self.filterError(text):
            #            if self.verbose:
            #                print "   ignore %s %s" % (prefix, text)
            return
        self.log_error += 1
        print("METADATA ERROR: %s %s" % (prefix, text))

    def fuzzFile(self, fuzz):

        failure = False
        while fuzz.nb_extract < MAX_NB_EXTRACT:
            if SLEEP_SEC:
                sleep(SLEEP_SEC)
            self.log_error = 0
            fatal_error = False
            try:
                failure = limitedMemory(MEMORY_LIMIT, fuzz.extract)
                prefix = fuzz.prefix
            except KeyboardInterrupt:
                try:
                    failure = (
                        input("Keep current file (y/n)?").strip() == "y")
                except (KeyboardInterrupt, EOFError):
                    print()
                    failure = False
                prefix = "interrupt"
                fatal_error = True
            except MemoryError:
                print("MEMORY ERROR!")
                failure = True
                prefix = "memory"
            except Exception as err:
                print("EXCEPTION (%s): %s" % (err.__class__.__name__, err))
                failure = True
                prefix = "exception"
            if fatal_error:
                break
            if failure is None:
                if fuzz.tryUndo():
                    failure = False
                elif fuzz.is_original:
                    print(
                        "    Warning: Unsupported file format: remove %s from test suite" % fuzz.filename)
                    self.filedb.remove(fuzz.filename)
                    return True
            if failure is None:
                break
            if failure:
                break
            if fuzz.acceptTruncate():
                if randint(0, TRUNCATE_RATE - 1) == 0:
                    fuzz.truncate()
                else:
                    fuzz.mangle()
            else:
                fuzz.mangle()

        # Process error
        if failure:
            fuzz.keepFile(prefix)
            self.nb_error += 1
        fuzz.sumUp()
        return (not fatal_error)

    def init(self):
        # Setup log
        self.nb_error = 0
        hachoir_logger.use_print = False
        hachoir_logger.on_new_message = self.newLog

        # Load file DB
        self.filedb = []
        for directory in self.filedb_dirs:
            new_files = glob(path.join(directory, "*.*"))
            self.filedb.extend(new_files)
        if not self.filedb:
            print("Empty directories: %s" % self.filedb_dirs)
            exit(1)

        # Create error directory
        try:
            mkdir(self.error_dir)
        except OSError as err:
            if err[0] == EEXIST:
                pass

    def run(self):
        self.init()
        try:
            while True:
                test_file = random_choice(self.filedb)
                print("[+] %s error -- test file: %s" %
                      (self.nb_error, test_file))
                fuzz = FileFuzzer(self, test_file)
                ok = self.fuzzFile(fuzz)
                if not ok:
                    break
        except KeyboardInterrupt:
            print("Stop")


def main():
    # Read command line argument
    if len(argv) < 2:
        print("usage: %s directory [directory2 ...]" % argv[0], file=stderr)
        exit(1)
    test_dirs = [path.normpath(path.expanduser(item)) for item in argv[1:]]

    # Directory is current directory?
    err_dir = path.join(getcwd(), "error")

    # Nice
    nice(19)

    fuzzer = Fuzzer(test_dirs, err_dir)
    fuzzer.run()


if __name__ == "__main__":
    main()



from hachoir.parser import createParser
from hachoir.editor import createEditor
from hachoir.field import writeIntoFile

parser = createParser("file.zip")
with parser:
    editor = createEditor(parser)
    editor["end_central_directory/comment"].value = "new comment"
    writeIntoFile(editor, "file_comment.zip")



#!/usr/bin/env python3
"""
Test hachoir-metadata using the testcase.
"""

from hachoir.parser import createParser
from hachoir.core.language import Language
from hachoir.metadata import extractMetadata
from hachoir.metadata.timezone import createTimezone
from hachoir.test import setup_tests
from datetime import date, timedelta, datetime
import os
import subprocess
import sys
import unittest

DATADIR = os.path.join(os.path.dirname(__file__), 'files')
PROGRAM = os.path.join(os.path.dirname(__file__), "..", "hachoir-metadata")


class TestMetadata(unittest.TestCase):
    verbose = False

    def extract(self, filename):
        if self.verbose:
            print("[+] Test %s:" % filename)
            sys.stdout.write("  - Create parser: ")
            sys.stdout.flush()

        fullname = os.path.join(DATADIR, filename)
        parser = createParser(fullname)
        if not parser:
            self.fail("unable to create parser\n")

        with parser:
            if self.verbose:
                sys.stdout.write("ok\n")
                sys.stdout.write("  - Create metadata: ")
                sys.stdout.flush()

            metadata = extractMetadata(parser, 1.0)
            if not metadata:
                self.fail("unable to create parser\n")

        if self.verbose:
            sys.stdout.write("ok\n")
        return metadata

    def test_dict_output(self):
        required_meta = {'Common': {'duration': '0:00:17.844000',
                                    'creation_date': '2006-08-16 11:04:36',
                                    'producer': 'libebml v0.7.7 + libmatroska v0.8.0',
                                    'mime_type': 'video/x-matroska',
                                    'endian': 'Big endian'
                                    },
                         'video[1]': {'language': 'French',
                                      'width': '384',
                                      'height': '288',
                                      'compression': 'V_MPEG4/ISO/AVC'
                                      },
                         'audio[1]': {'title': 'travail = aliénation (extrait)',
                                      'language': 'French',
                                      'nb_channel': '1',
                                      'sample_rate': '44100.0',
                                      'compression': 'A_VORBIS'
                                      }
                         }
        with createParser(os.path.join(DATADIR, "flashmob.mkv")) as parser:
            extractor = extractMetadata(parser)
        meta = extractor.exportDictionary(human=False)

        self.assertEqual(required_meta, meta)

    def test_plaintext_output(self):
        expected_plaintext = [['Video stream:',
                               '- Language: French',
                               '- Image width: 384 pixels',
                               '- Image height: 288 pixels',
                               '- Compression: V_MPEG4/ISO/AVC'],
                              ['Audio stream:',
                               '- Title: travail = aliénation (extrait)',
                               '- Language: French',
                               '- Channel: mono',
                               '- Sample rate: 44.1 kHz',
                               '- Compression: A_VORBIS']]
        with createParser(os.path.join(DATADIR, "flashmob.mkv")) as parser:
            extractor = extractMetadata(parser)
        groups = [g.exportPlaintext() for g in extractor.iterGroups()]
        self.assertEqual(groups, expected_plaintext)

    def check_attr(self, metadata, name, value):
        if self.verbose:
            sys.stdout.write("  - Check metadata %s=%s: " %
                             (name, repr(value)))

        if not isinstance(value, (list, tuple)):
            value = [value]

        # Has subgroup? (eg. "audio/sample_rate")
        if "/" in name:
            group, name = name.split("/", 1)
            if group not in metadata:
                if self.verbose:
                    sys.stdout.write("no group \"%s\"!\n" % group)
                return False
            metadata = metadata[group]

        # Has asked attribute?
        if not metadata.has(name):
            if self.verbose:
                sys.stdout.write("no attribute \"%s\"!\n" % name)
            return False

        # Read value
        reads = metadata.getValues(name)

        # Check value
        if len(reads) != len(value):
            if self.verbose:
                sys.stdout.write("wrong len (%s instead of %s)!\n"
                                 % (len(reads), len(value)))
            return False
        values = value
        for index, value in enumerate(values):
            read = reads[index]

            # Check type
            if type(read) != type(value) \
                    and not(isinstance(value, int) and isinstance(value, int)):
                if self.verbose:
                    sys.stdout.write("wrong type (%s instead of %s)!\n"
                                     % (type(read).__name__, type(value).__name__))
                return False

            # Check value
            if value != read:
                if self.verbose:
                    sys.stdout.write("wrong value %s (%r instead of %r)!\n"
                                     % (index, read, value))
                return False
        if self.verbose:
            sys.stdout.write("ok\n")
        return True

    def test_png(self):
        metadata = self.extract("logo-kubuntu.png")
        self.check_attr(metadata, "bits_per_pixel", 32)
        self.check_attr(metadata, "creation_date",
                        datetime(2006, 5, 26, 9, 41, 46))
        self.check_attr(metadata, "mime_type", "image/png")

    def test_wav(self):
        metadata = self.extract("kde_click.wav")
        self.check_attr(metadata, "producer", "Sound Forge 4.5")
        self.check_attr(metadata, "creation_date", date(2001, 2, 21))
        self.check_attr(metadata, "duration", timedelta(microseconds=19546))
        self.check_attr(metadata, "bit_rate", 705600)
        self.check_attr(metadata, "sample_rate", 22050)

    def test_gzip(self):
        metadata = self.extract("test.txt.gz")
        self.check_attr(metadata, "file_size", 99)
        self.check_attr(metadata, "compr_size", 90)
        self.check_attr(metadata, "last_modification",
                        datetime(2006, 7, 29, 12, 20, 44))
        self.check_attr(metadata, "os", "Unix")
        self.check_attr(metadata, "compression", "deflate")

    def test_mp3(self):
        metadata = self.extract("sheep_on_drugs.mp3")
        self.check_attr(metadata, "format_version", "MPEG version 1 layer III")
        self.check_attr(metadata, "author", "Sheep On Drugs")
        self.check_attr(
            metadata, "comment", "Stainless Steel Provider is compilated to the car of Twinstar.")

    def test_png2(self):
        metadata = self.extract("png_331x90x8_truncated.png")
        self.check_attr(metadata, "width", 331)
        self.check_attr(metadata, "creation_date",
                        datetime(2006, 5, 26, 9, 41, 46))
        self.check_attr(metadata, "mime_type", "image/png")
        self.check_attr(metadata, "endian", "Big endian")

    def test_mkv(self):
        metadata = self.extract("flashmob.mkv")
        self.check_attr(metadata, "copyright",
                        "© dadaprod, licence Creative Commons by-nc-sa 2.0 fr")
        self.check_attr(metadata, "video[1]/width", 384)
        self.check_attr(metadata, "video[1]/language", Language('fre'))
        self.check_attr(metadata, "duration", timedelta(
            seconds=17, milliseconds=844))

    def test_mkv2(self):
        meta = self.extract("10min.mkv")
        self.check_attr(meta, "duration", timedelta(minutes=10))
        self.check_attr(meta, "producer", ["x264", "Haali Matroska Writer b0"])
        self.check_attr(meta, "video[1]/width", 384)
        self.check_attr(meta, "video[1]/height", 288)
        self.check_attr(meta, "video[1]/compression", "V_MPEG4/ISO/AVC")

    def test_ico(self):
        meta = self.extract("wormux_32x32_16c.ico")
        self.check_attr(meta, "image[0]/width", 16)
        self.check_attr(meta, "image[0]/height", 16)
        self.check_attr(meta, "image[0]/bits_per_pixel", 32)
        self.check_attr(meta, "image[0]/compression", "Uncompressed (RGB)")

    def test_au(self):
        meta = self.extract("audio_8khz_8bit_ulaw_4s39.au")
        self.check_attr(meta, "mime_type", "audio/basic")
        self.check_attr(meta, "nb_channel", 1)
        self.check_attr(meta, "bits_per_sample", 8)
        self.check_attr(meta, "bit_rate", 64096)
        self.check_attr(meta, "sample_rate", 8012)
        self.check_attr(meta, "compression", "8-bit ISDN u-law")
        self.check_attr(meta, "comment", "../tmp/temp.snd")
        self.check_attr(meta, "duration", timedelta(
            seconds=4, microseconds=391538))

    def test_xcf(self):
        meta = self.extract("cross.xcf")
        self.check_attr(meta, "comment", "Created with The GIMP")
        self.check_attr(meta, "width", 61)
        self.check_attr(meta, "height", 72)
        self.check_attr(meta, "compression", "RLE")
        self.check_attr(meta, "mime_type", "image/x-xcf")

    def test_tar(self):
        meta = self.extract("small_text.tar")
        self.check_attr(meta, "file[0]/filename", "dummy.txt")
        self.check_attr(meta, "file[0]/file_size", 62)
        self.check_attr(meta, "file[1]/file_attr", "-rwxr-xr-x (755)")
        self.check_attr(meta, "file[1]/last_modification",
                        datetime(2006, 10, 1, 13, 9, 3))
        self.check_attr(meta, "file[2]/file_type", "Normal disk file")

    def test_bmp(self):
        meta = self.extract("kde_haypo_corner.bmp")
        self.check_attr(meta, "width", 189)
        self.check_attr(meta, "nb_colors", 70)
        self.check_attr(meta, "compression", "Uncompressed")
        self.check_attr(meta, "mime_type", "image/x-ms-bmp")

    def test_avi(self):
        metadata = self.extract("smallville.s03e02.avi")
        self.check_attr(metadata, "duration", timedelta(
            minutes=44, seconds=1, microseconds=141141))
        self.check_attr(metadata, "producer",
                        "VirtualDubMod 1.5.10.1 (build 2366/release)")
        self.check_attr(metadata, "video/width", 640)
        self.check_attr(metadata, "video/height", 352)
        self.check_attr(metadata, "video/compression",
                        'XviD MPEG-4 (fourcc:"xvid")')
        self.check_attr(metadata, "video/frame_rate", 23.976)
        self.check_attr(metadata, "audio[1]/nb_channel", 2)
        self.check_attr(metadata, "audio[1]/sample_rate", 48000)
        self.check_attr(metadata, "audio[1]/compression", "MPEG Layer 3")

    def test_mp3_2(self):
        meta = self.extract("08lechat_hq_fr.mp3")
        self.check_attr(meta, "album", ["Arte Radio", "Chat Broodthaers"])
        self.check_attr(
            meta, "url", "Liens direct ARTE Radio: www.arteradio.com/son.html?473")
        self.check_attr(meta, "creation_date", date(2003, 1, 1))
        self.check_attr(meta, "producer", "www.arteradio.com")
        self.check_attr(meta, "sample_rate", 44100)
        self.check_attr(meta, "bit_rate", 128000)

    def test_jpeg(self):
        meta = self.extract("jpeg.exif.photoshop.jpg")
        self.check_attr(meta, "producer", ["Adobe Photoshop 7.0"])
        self.check_attr(meta, "width", 124)
        self.check_attr(meta, "compression", "JPEG (Progressive)")
        self.check_attr(meta, "creation_date",
                        datetime(2006, 6, 28, 14, 51, 9))

    def test_ogg_vorbis(self):
        meta = self.extract("interlude_david_aubrun.ogg")
        self.check_attr(meta, "title", "interlude symbiosys1")
        self.check_attr(meta, "artist", "david aubrun")
        self.check_attr(meta, "duration", timedelta(
            minutes=1, seconds=12, microseconds=19592))
        self.check_attr(meta, "audio[1]/nb_channel", 2)
        self.check_attr(meta, "audio[1]/format_version", "Vorbis version 0")
        self.check_attr(meta, "audio[1]/sample_rate", 44100)
        self.check_attr(meta, "mime_type", "audio/vorbis")

    def test_flv(self):
        meta = self.extract("breakdance.flv")
        self.check_attr(meta, "audio/sample_rate", 22050)
        self.check_attr(meta, "duration", timedelta(
            seconds=46, milliseconds=942))
        self.check_attr(meta, "producer", [
                        "YouTube, Inc.", "YouTube Metadata Injector."])

    def test_wmv(self):
        meta = self.extract("matrix_ping_pong.wmv")
        self.check_attr(meta, "title", "欽ちゃん＆香取慎吾の全日本仮装大賞")
        self.check_attr(meta, "duration", timedelta(
            minutes=1, seconds=43, milliseconds=900))
        self.check_attr(meta, "creation_date", datetime(
            2003, 6, 16, 7, 57, 23, 235000))
        self.check_attr(meta, "audio[1]/sample_rate", 8000)
        self.check_attr(meta, "audio[1]/bits_per_sample", 16)
        self.check_attr(meta, "audio[1]/compression",
                        "Windows Media Audio V7 / V8 / V9")
        self.check_attr(meta, "video[1]/width", 200)
        self.check_attr(meta, "video[1]/height", 150)
        self.check_attr(meta, "video[1]/bits_per_pixel", 24)

    def test_jpeg_2(self):
        meta = self.extract("usa_railroad.jpg")
        # Check IPTC parser
        self.check_attr(meta, "author", "Ian Britton")
        self.check_attr(meta, "copyright", "FreeFoto.com")
        self.check_attr(meta, "thumbnail_size", 4196),
        self.check_attr(meta, "iso_speed_ratings", 160),
        self.check_attr(meta, "exif_version", "0220"),
        self.check_attr(meta, "date_time_original", "2003:08:06 17:52:54"),
        self.check_attr(meta, "date_time_digitized", "2003:08:06 17:52:54"),
        self.check_attr(meta, "compressed_bits_per_pixel", 3.2),
        self.check_attr(meta, "aperture_value", 6),
        self.check_attr(meta, "exposure_bias_value", -0.5),
        self.check_attr(meta, "focal_length", 137),
        self.check_attr(meta, "flashpix_version", u"0100"),
        self.check_attr(meta, "focal_plane_x_resolution", 1322.0),
        self.check_attr(meta, "focal_plane_y_resolution", 1322.0),
        self.check_attr(meta, "focal_length_in_35mm_film", 205),

    def test_tga(self):
        meta = self.extract("hero.tga")
        self.check_attr(meta, "width", 320)
        self.check_attr(meta, "bits_per_pixel", 8)
        self.check_attr(meta, "nb_colors", 256)
        self.check_attr(meta, "compression", "8-bit uncompressed")

    def test_aifc(self):
        meta = self.extract("25min.aifc")
        self.check_attr(meta, "duration", timedelta(minutes=25, seconds=33))
        self.check_attr(meta, "nb_channel", 2)
        self.check_attr(meta, "sample_rate", 44100)
        self.check_attr(meta, "bit_rate", 1411200)
        self.check_attr(meta, "bits_per_sample", 16)
        self.check_attr(meta, "compression", "Little-endian, no compression")

    def test_wav_2(self):
        meta = self.extract("ladouce_1h15.wav")
        self.check_attr(meta, "duration", timedelta(
            hours=1, minutes=16, seconds=32, microseconds=516032))
        self.check_attr(meta, "nb_channel", 6)
        self.check_attr(meta, "sample_rate", 44100)
        self.check_attr(meta, "bits_per_sample", 32)
        self.check_attr(meta, "compression", "IEEE Float")
        self.check_attr(meta, "bit_rate", 8467200)

    def test_pcx(self):
        meta = self.extract("lara_croft.pcx")
        self.check_attr(meta, "width", 320)
        self.check_attr(meta, "nb_colors", 256)
        self.check_attr(meta, "compression", "Run-length encoding (RLE)")

    def test_sxw(self):
        meta = self.extract("hachoir.org.sxw")
        self.check_attr(meta, "mime_type", "application/vnd.sun.xml.writer")
        self.check_attr(meta, "file[0]/file_size", 30)
        self.check_attr(meta, "file[1]/creation_date",
                        datetime(2007, 1, 22, 19, 8, 14))
        self.check_attr(meta, "file[2]/filename",
                        "Configurations2/accelerator/current.xml")
        self.check_attr(meta, "file[2]/compression", "Deflate")

    def test_rm(self):
        meta = self.extract("firstrun.rm")
        self.check_attr(meta, "duration", timedelta(
            seconds=17, milliseconds=66))
        self.check_attr(meta, "creation_date",
                        datetime(2000, 6, 14, 10, 3, 18))
        self.check_attr(meta, "copyright", "©2000 RealNetworks")
        self.check_attr(meta, "producer",
                        "RealProducer Plus 6.1.0.153 Windows")
        self.check_attr(meta, "stream[0]/mime_type", "audio/x-pn-realaudio")
        self.check_attr(meta, "stream[0]/bit_rate", 32148)
        self.check_attr(meta, "stream[0]/title", "Audio Stream")
        self.check_attr(meta, "mime_type", "audio/x-pn-realaudio")
        self.check_attr(meta, "bit_rate", 32348)
        self.check_attr(meta, "stream[1]/bit_rate", 200)

    def test_ttf(self):
        meta = self.extract("deja_vu_serif-2.7.ttf")
        self.check_attr(meta, "title", "DejaVu Serif")
        self.check_attr(meta, "author", "DejaVu fonts team")
        self.check_attr(meta, "version", "2.7")
        self.check_attr(meta, "creation_date",
                        datetime(2006, 7, 6, 17, 29, 52))
        self.check_attr(meta, "last_modification",
                        datetime(2006, 7, 6, 17, 29, 52))
        self.check_attr(meta, "copyright", [
            "Copyright (c) 2003 by Bitstream, Inc. All Rights Reserved.\nDejaVu changes are in public domain",
            "http://dejavu.sourceforge.net/wiki/index.php/License"])
        self.check_attr(meta, "url", "http://dejavu.sourceforge.net")
        self.check_attr(meta, "comment", [
            "Smallest readable size in pixels: 8 pixels",
            "Font direction: Mixed directional"])

    def test_exe_pe(self):
        meta = self.extract("twunk_16.exe")
        self.check_attr(meta, "title", [
            "Twain_32.dll Client's 16-Bit Thunking Server",
            "Twain Thunker"])
        self.check_attr(meta, "author", "Twain Working Group")
        self.check_attr(meta, "version", "1,7,0,0")
        self.check_attr(meta, "format_version",
                        "New-style executable: Dynamic-link library (DLL)")

    def test_torrent(self):
        meta = self.extract("debian-31r4-i386-binary-1.iso.torrent")
        self.check_attr(meta, "filename", "debian-31r4-i386-binary-1.iso")
        self.check_attr(
            meta, "url", "http://bttracker.acc.umu.se:6969/announce")
        self.check_attr(meta, "file_size", 669775872)
        self.check_attr(meta, "creation_date",
                        datetime(2006, 11, 16, 21, 44, 37))

    def test_jpeg_3(self):
        meta = self.extract("green_fire.jpg")
        self.check_attr(meta, 'height', 64)
        self.check_attr(meta, 'bits_per_pixel', 32)
        self.check_attr(
            meta, 'comment', ("Intel(R) JPEG Library, version 1,5,4,36", "JPEG quality: 80%"))

    def test_mp3_3(self):
        meta = self.extract("marc_kravetz.mp3")
        self.check_attr(meta, 'creation_date', datetime(
            2007, 7, 19, 9, 3, 57, tzinfo=createTimezone(2)))
        self.check_attr(meta, 'sample_rate', 48000)
        self.check_attr(meta, 'compr_rate', 12.0)
        self.check_attr(
            meta, 'album', "France Culture - Le portrait du jour par Marc Kravetz")
        self.check_attr(meta, 'author', "Marc Kravetz")
        self.check_attr(meta, 'duration', timedelta(0, 2, 400000))
        self.check_attr(meta, 'bit_rate', 128000)
        self.check_attr(meta, 'track_number', 32)
        self.check_attr(meta, 'bits_per_sample', 16)
        self.check_attr(meta, 'copyright', "Radio France")
        self.check_attr(meta, 'format_version', "MPEG version 1 layer III")

    def test_mov(self):
        meta = self.extract("pentax_320x240.mov")
        self.check_attr(meta, 'width', 320)
        self.check_attr(meta, 'height', 240)
        self.check_attr(meta, 'duration', timedelta(0, 4, 966667))
        self.check_attr(meta, 'creation_date',
                        datetime(2005, 8, 11, 14, 3, 54))
        self.check_attr(meta, 'last_modification',
                        datetime(2005, 8, 11, 14, 3, 54))

    def test_jpeg_gps(self):
        meta = self.extract("gps.jpg")
        self.check_attr(meta, 'altitude', 78.0)
        self.check_attr(meta, 'creation_date',
                        datetime(2003, 5, 24, 22, 29, 14))
        self.check_attr(meta, 'latitude', 35.616019444444447)
        self.check_attr(meta, 'longitude', 139.69731666666667)
        self.check_attr(meta, 'camera_model', 'A5301T')
        self.check_attr(meta, 'camera_manufacturer', 'KDDI-TS')

    def test_ani(self):
        meta = self.extract("angle-bear-48x48.ani")
        self.check_attr(meta, 'title', "Angel Bear")
        self.check_attr(
            meta, 'artist', "Copyright ©Loraine Wauer-Ferus http://www.billybear4kids.com")
        self.check_attr(meta, 'frame_rate', 4.0)

    def test_flac(self):
        meta = self.extract("hotel_california.flac")
        self.check_attr(meta, 'title', "Hotel California")
        self.check_attr(meta, 'artist', "The Eagles")
        self.check_attr(meta, 'duration', timedelta(
            seconds=51, microseconds=512834))
        self.check_attr(meta, 'nb_channel', 2)
        self.check_attr(meta, 'sample_rate', 44100)
        self.check_attr(meta, 'bits_per_sample', 16)
        self.check_attr(meta, 'producer', 'reference libFLAC 1.1.2 20050205')

    def test_doc(self):
        meta = self.extract("radpoor.doc")
        self.check_attr(meta, 'title', "\u062a\u0633\u062a")
        self.check_attr(meta, 'author', 'Soroosh Radpoor')
        self.check_attr(meta, 'creation_date', datetime(2008, 9, 2, 16, 8, 30))

    def test_mpeg4(self):
        meta = self.extract("quicktime.mp4")
        self.check_attr(meta, 'width', 190)
        self.check_attr(meta, 'height', 240)
        self.check_attr(meta, 'creation_date',
                        datetime(2005, 10, 28, 17, 46, 46))
        self.check_attr(meta, 'mime_type', 'video/mp4')

    def test_tiff(self):
        meta = self.extract("sample.tif")
        self.check_attr(meta, 'width', 1600)
        self.check_attr(meta, 'height', 2100)
        self.check_attr(meta, 'width_dpi', 200)
        self.check_attr(meta, 'height_dpi', 200)
        self.check_attr(meta, 'producer', 'mieconvert')

    def test_cr2(self):
        meta = self.extract("canon.raw.cr2")
        self.check_attr(meta, 'width', 5184)
        self.check_attr(meta, 'height', 3456)
        self.check_attr(meta, 'creation_date',
                        datetime(2015, 11, 15, 13, 35, 29))
        self.check_attr(meta, 'date_time_original',
                        datetime(2015, 11, 15, 13, 35, 29))
        self.check_attr(meta, 'camera_manufacturer', 'Canon')
        self.check_attr(meta, 'camera_model', 'Canon EOS REBEL T5i')


class TestMetadataCommandLine(unittest.TestCase):

    def test_metadata(self):
        filename = os.path.join(DATADIR, 'gps.jpg')
        args = [sys.executable, PROGRAM, filename]
        proc = subprocess.Popen(args,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.STDOUT)
        stdout, _ = proc.communicate()
        stdout = stdout.decode('ascii', 'replace').strip()
        self.assertEqual(stdout, """Metadata:
- Title: 030524_2221~01
- Image width: 144 pixels
- Image height: 176 pixels
- Image orientation: Horizontal (normal)
- Bits/pixel: 24
- Pixel format: YCbCr
- Creation date: 2003-05-24 22:29:14
- Latitude: 35.61601944444445
- Longitude: 139.69731666666667
- Altitude: 78.0 meters
- Camera model: A5301T
- Camera manufacturer: KDDI-TS
- Compression: JPEG (Baseline)
- EXIF version: 0220
- Date-time original: 2003-05-24 22:21:01
- Flashpix version: 0100
- Comment: JPEG quality: 90%
- MIME type: image/jpeg
- Endianness: Big endian""")


if __name__ == "__main__":
    setup_tests()
    unittest.main()



#!/usr/bin/env python3
from hachoir.grep import Grep
from hachoir.parser import createParser
from hachoir.test import setup_tests
import os.path
import subprocess
import sys
import unittest

DATADIR = os.path.join(os.path.dirname(__file__), "files")
GEORGIA_CAB = os.path.join(DATADIR, 'georgia.cab')
PROGRAM = os.path.join(os.path.dirname(__file__), "..", "hachoir-grep")


class TestGrepClass(unittest.TestCase):

    def test_grep(self):
        fields = []

        class TestGrep(Grep):

            def onMatch(self, field):
                fields.append(field)

        parser = createParser(GEORGIA_CAB)
        with parser:
            grep = TestGrep()
            grep.grep(parser)
        fields = [(field.absolute_address, field.path, field.value)
                  for field in fields]
        self.assertEqual(fields,
                         [(0, '/magic', 'MSCF'),
                          (480, '/file[0]/filename', 'fontinst.inf'),
                             (712, '/file[1]/filename', 'Georgiaz.TTF'),
                             (944, '/file[2]/filename', 'Georgiab.TTF'),
                             (1176, '/file[3]/filename', 'Georgiai.TTF'),
                             (1408, '/file[4]/filename', 'Georgia.TTF'),
                             (1632, '/file[5]/filename', 'fontinst.exe')])


class TestGrepCommandLine(unittest.TestCase):

    def test_grep(self):
        args = [sys.executable, PROGRAM, "--all", "--path", GEORGIA_CAB]
        proc = subprocess.Popen(args,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.STDOUT)
        stdout, _ = proc.communicate()
        stdout = stdout.decode('ascii', 'replace')
        self.assertEqual(stdout, """
0:/magic:MSCF
60:/file[0]/filename:fontinst.inf
89:/file[1]/filename:Georgiaz.TTF
118:/file[2]/filename:Georgiab.TTF
147:/file[3]/filename:Georgiai.TTF
176:/file[4]/filename:Georgia.TTF
204:/file[5]/filename:fontinst.exe
""".lstrip())


if __name__ == "__main__":
    setup_tests()
    unittest.main()



#!/usr/bin/env python3
from math import log


class Entropy:

    def __init__(self):
        self.frequence = dict((index, 0) for index in range(0, 256))
        self.count = 0

    def readBytes(self, bytes):
        for byte in bytes:
            self.frequence[byte] = self.frequence[byte] + 1
        self.count += len(bytes)
        return self

    def compute(self):
        h = 0
        for value in self.frequence.values():
            if not value:
                continue
            p_i = float(value) / self.count
            h -= p_i * log(p_i, 2)
        return h

from time import time
from sys import stderr


class EntropyFile(Entropy):

    def __init__(self):
        Entropy.__init__(self)
        self.progress_time = 1.0
        self.buffer_size = 4096

    def displayProgress(self, percent):
        print("Progress: %.1f%%" % percent, file=stderr)

    def readStream(self, stream, streamsize=None):
        # Read stream size
        if streamsize is None:
            stream.seek(0, 2)
            streamsize = stream.tell()
        if streamsize <= 0:
            raise ValueError("Empty stream")

        # Read stream content
        stream.seek(0, 0)
        next_msg = time() + self.progress_time
        while True:
            if next_msg <= time():
                self.displayProgress(stream.tell() * 100.0 / streamsize)
                next_msg = time() + self.progress_time
            raw = stream.read(self.buffer_size)
            if not raw:
                break
            self.readBytes(raw)
        return self

    def readFile(self, filename):
        stream = open(filename, 'rb')
        self.readStream(stream)
        return self


def main():
    from sys import argv, exit
    if len(argv) != 2:
        print("usage: %s filename" % argv[0], file=stderr)
        exit(1)
    entropy = EntropyFile()
    entropy.readFile(argv[1])
    print("Entropy: %.4f bit/byte" % entropy.compute())
    exit(0)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import os.path

def writeReadme(out):
    from hachoir.parser.parser_list import HachoirParserList

    # Write parser list
    format = "rest"
    if format == "rest":
        print("Parser list", file=out)
        print("===========", file=out)
        print(file=out)
    HachoirParserList().print_(out=out, format=format)

def main():
    path = os.path.dirname(__file__)
    filename = os.path.join(path, 'parser_list.rst')
    with open(filename, 'w') as readme:
        writeReadme(readme)
    print("%s regenerated" % filename)

if __name__ == "__main__":
    main()




from sys import platform

if platform == 'win32':
    from win32process import (GetCurrentProcess, SetPriorityClass,
                              BELOW_NORMAL_PRIORITY_CLASS)

    def beNice():
        process = GetCurrentProcess()
        # FIXME: Not supported on Windows 95/98/Me/NT: ignore error?
        # which error?
        SetPriorityClass(process, BELOW_NORMAL_PRIORITY_CLASS)

    OS_ERRORS = (OSError, WindowsError)
else:
    from os import nice

    def beNice():
        nice(19)

    OS_ERRORS = OSError

try:
    import sha

    def generateUniqueID(data):
        return sha.new(data).hexdigest()
except ImportError:
    def generateUniqueID(data):
        generateUniqueID.sequence += 1
        return generateUniqueID.sequence
    generateUniqueID.sequence = 0


def getFilesize(file):
    file.seek(0, 2)
    size = file.tell()
    file.seek(0, 0)
    return size



#!/usr/bin/env python3
"""
Test hachoir-parser using the testcase.
"""

from hachoir.core.error import error
from hachoir.stream import StringInputStream
from hachoir.parser import createParser, HachoirParserList, ValidateError
from hachoir.test import setup_tests
from array import array
from datetime import datetime
import random
import os
import sys
import unittest

DATADIR = os.path.join(os.path.dirname(__file__), 'files')


class TestParsers(unittest.TestCase):
    verbose = False

    def parse(self, filename):
        if self.verbose:
            print("[+] Test %s:" % filename)
            sys.stdout.write("  - Create parser: ")
            sys.stdout.flush()

        fullname = os.path.join(DATADIR, filename)
        parser = createParser(fullname)
        if not parser:
            self.fail("unable to create parser")
        self.addCleanup(parser.close)

        if self.verbose:
            sys.stdout.write("ok\n")
        return parser

    def checkValue(self, parser, path, value):
        if self.verbose:
            sys.stdout.write("  - Check field %s.value=%s (%s): "
                             % (path, repr(value), value.__class__.__name__))
            sys.stdout.flush()
        read = parser[path].value
        self.assertEqual(read, value,
                         "wrong value (%s, %s)"
                         % (repr(read), read.__class__.__name__))

    def checkDisplay(self, parser, path, value):
        if self.verbose:
            sys.stdout.write("  - Check field %s.display=%s (%s): "
                             % (path, repr(value), value.__class__.__name__))
            sys.stdout.flush()
        read = parser[path].display
        self.assertEqual(read, value,
                         "wrong value (%s, %s)"
                         % (repr(read), read.__class__.__name__))

    def checkDesc(self, parser, path, value):
        if self.verbose:
            sys.stdout.write("  - Check field %s.description=%s (%s): "
                             % (path, repr(value), value.__class__.__name__))
            sys.stdout.flush()
        read = parser[path].description
        self.assertEqual(read, value,
                         "wrong value (%s, %s)"
                         % (repr(read), read.__class__.__name__))

    def checkNames(self, parser, path, names):
        if self.verbose:
            sys.stdout.write("  - Check field names %s=(%s) (%u): "
                             % (path, ", ".join(names), len(names)))
            sys.stdout.flush()
        fieldset = parser[path]
        if len(fieldset) != len(names):
            self.fail("invalid length (%u)" % len(fieldset))
        names = list(names)
        read = list(field.name for field in fieldset)
        self.assertEqual(names, read,
                         "wrong names (%s)" % ", ".join(read))

    def test_3ds(self):
        parser = self.parse("yellowdude.3ds")
        self.checkValue(parser, "/main/version/version", 2)
        self.checkNames(parser, "/main",
                        ("type", "size", "version", "obj_mat"))

    def test_png(self):
        parser = self.parse("logo-kubuntu.png")
        self.checkValue(parser, "/header/width", 331)
        self.checkValue(parser, "/time/second", 46)

    def test_wav(self):
        parser = self.parse("kde_click.wav")
        self.checkValue(parser, "/info/producer/text", "Sound Forge 4.5")
        self.checkValue(parser, "/format/sample_per_sec", 22050)

    def test_mbr(self):
        parser = self.parse("mbr_linux_and_ext")
        self.checkValue(parser, "/mbr/header[1]/size", 65545200)
        self.checkDisplay(parser, "/mbr/signature", "0xaa55")

    def test_mkv(self):
        parser = self.parse("flashmob.mkv")
        self.checkValue(parser, "/Segment[0]/Cues/CuePoint[1]/CueTrackPositions[0]"
                        + "/CueClusterPosition/cluster/BlockGroup[14]/Block/block/timecode", 422)
        self.checkValue(parser, "/Segment[0]/Tags[0]/Tag[0]/SimpleTag[3]/TagString/unicode",
                        "\xa9 dadaprod, licence Creative Commons by-nc-sa 2.0 fr")

    def test_mkv2(self):
        parser = self.parse("10min.mkv")
        self.checkValue(parser, "/Segment[0]/size", None)
        self.checkValue(
            parser, "/Segment[0]/Tracks[0]/TrackEntry[0]/CodecID/string", "V_MPEG4/ISO/AVC")

    def test_ico(self):
        parser = self.parse("wormux_32x32_16c.ico")
        self.checkValue(parser, "icon_header[0]/height", 16)
        self.checkValue(parser, "icon_data[0]/header/hdr_size", 40)

    def test_cda(self):
        parser = self.parse("cd_0008_5C48_1m53s.cda")
        self.checkValue(parser, "filesize", 36)
        self.checkValue(parser, "/cdda/track_no", 4)
        self.checkDisplay(parser, "/cdda/disc_serial", "0008-5C48")
        self.checkValue(parser, "/cdda/rb_length/second", 53)

    def test_mp3(self):
        parser = self.parse("sheep_on_drugs.mp3")
        self.checkValue(parser, "/id3v2/field[6]/content/text",
                        'Stainless Steel Provider is compilated to the car of Twinstar.')
        self.checkValue(parser, "/frames/frame[0]/use_padding", False)

    def test_au(self):
        parser = self.parse("audio_8khz_8bit_ulaw_4s39.au")
        self.checkValue(parser, "info", "../tmp/temp.snd")
        self.checkDisplay(parser, "codec", "8-bit ISDN u-law")

    def test_gzip(self):
        parser = self.parse("test.txt.gz")
        self.checkValue(parser, "filename", "test.txt")

    def test_mp3_steganography(self):
        parser = self.parse("steganography.mp3")
        self.checkValue(parser, "/frames/padding[0]", b"misc est un canard\r")
        self.checkDesc(parser, "/frames", 'Frames: Variable bit rate (VBR)')
        self.checkDesc(
            parser, "/frames/frame[1]", 'MPEG-1 layer III, 160.0 Kbit/sec, 44.1 kHz')

    def test_rpm(self):
        parser = self.parse("ftp-0.17-537.i586.rpm")
        self.checkValue(parser, "name", "ftp-0.17-537")
        self.checkValue(parser, "os", 1)
        self.checkValue(parser, "checksum/content_item[2]/value", 50823)
        self.checkValue(
            parser, "header/content_item[15]/value", "ftp://ftp.uk.linux.org/pub/linux/Networking/netkit")

    def test_jpeg(self):
        parser = self.parse("jpeg.exif.photoshop.jpg")
        self.checkValue(parser, "app0/content/jfif", "JFIF\x00")
        self.checkValue(parser, "app0/content/ver_maj", 1)
        self.checkValue(parser, "photoshop/content/signature", "Photoshop 3.0")
        self.checkValue(parser, "photoshop/content/copyright_flag/content", 0)
        self.checkValue(parser, "exif/content/header", "Exif\0\0")
        self.checkValue(parser, "exif/content/version", 42)

    def test_tar(self):
        parser = self.parse("small_text.tar")
        self.checkDisplay(parser, "file[0]/name", '"dummy.txt"')
        self.checkDisplay(parser, "file[1]/mode", '"0000755"')
        self.checkDisplay(parser, "file[1]/type", 'Directory')
        self.checkDisplay(parser, "file[2]/devmajor", '(empty)')

    def test_rar(self):
        parser = self.parse("hachoir-core.rar")
        self.checkValue(parser, "archive_start/crc16", 0x77E1)
        self.checkValue(parser, "new_sub_block[0]/crc16", 0x2994)
        self.checkValue(parser, "file[0]/crc32", 0x4C6D13ED)
        self.checkValue(parser, "new_sub_block[1]/crc32", 0x34528E23)
        self.checkValue(parser, "file[1]/filename",
                        r".svn\prop-base\README.svn-base")
        self.checkValue(parser, "new_sub_block[1]/filename", 'ACL')
        # archive_end bad candidate for checking
        self.checkValue(parser, "new_sub_block[362]/crc32", 0x6C84C95E)

    def test_ace(self):
        parser = self.parse("hachoir-core.ace")
        self.checkValue(parser, "header/crc16", 0xA9BE)
        self.checkValue(parser, "file[0]/reserved", 0x4554)
        self.checkValue(parser, "file[1]/filename", r"hachoir_core\.svn")
        self.checkValue(parser, "file[2]/parameters", 0x000A)
        # End of archive, lots of work...
        # self.checkValue(parser, "new_recovery[0]/signature", "**ACE**")

    def test_bmp(self):
        parser = self.parse("kde_haypo_corner.bmp")
        self.checkValue(parser, "header/width", 189)
        self.checkValue(parser, "header/used_colors", 70)
        self.checkDesc(
            parser, "palette/color[1]", "RGB color: White (opacity: 0%)")
        self.checkValue(parser, "pixels/line[26]/pixel[14]", 28)

    def test_der(self):
        parser = self.parse("cacert_class3.der")
        self.checkDisplay(parser, "seq[0]/class", 'universal')
        self.checkDesc(parser, "seq[0]/seq[0]/seq[1]/obj_id[0]",
                       "Object identifier: 1.2.840.113549.1.1.4")
        self.checkValue(
            parser, "seq[0]/seq[0]/seq[2]/set[0]/seq[0]/print_str[0]/value", "Root CA")
        self.checkValue(
            parser, "seq[0]/seq[0]/seq[2]/set[3]/seq[0]/ia5_str[0]/value", "support@cacert.org")
        self.checkValue(parser, "seq[0]/bit_str[0]/size", 513)

    def check_pyc(self, parser):
        self.checkValue(parser, "/content/consts/item[0]", 42)
        self.checkValue(parser, "/content/stack_size", 4)
        self.checkValue(
            parser, "/content/consts/item[1]", 2535301200456458802993406410752)
        self.checkValue(parser, "/content/consts/item[4]", 0.3j)
        self.checkValue(parser, "/content/consts/item[8]", b"abc")
        self.checkValue(parser, "/content/filename", b"pyc_example.py")

    def test_pyc_1_5(self):
        parser = self.parse("pyc_example_1.5.2_pyc.bin")
        self.check_pyc(parser)

    def test_pyc_2_2(self):
        parser = self.parse("pyc_example_2.2.3_pyc.bin")
        self.check_pyc(parser)

    def test_pyc_2_5(self):
        parser = self.parse("pyc_example_2.5c1_pyc.bin")
        self.check_pyc(parser)

    def check_pyc_37(self, parser):
        parser = self.parse("python.cpython-37.pyc.bin")
        self.checkValue(parser, "/content/consts/item[0]/name/text", "f")

    def test_java(self):
        parser = self.parse("ReferenceMap.class")
        self.checkValue(parser, "/minor_version", 3)
        self.checkValue(parser, "/major_version", 45)
        self.checkValue(parser, "/constant_pool_count", 326)
        self.checkValue(
            parser, "/constant_pool/constant_pool[324]/bytes", "([Ljava/lang/Object;Ljava/lang/Object;)V")
        self.checkValue(parser, "/super_class", 80)
        self.checkValue(parser, "/interfaces_count", 0)
        self.checkValue(parser, "/fields_count", 16)
        self.checkValue(
            parser, "/fields/fields[3]/attributes/attributes[0]/attribute_name_index", 93)
        self.checkValue(parser, "/methods_count", 31)
        self.checkValue(
            parser, "/methods/methods[30]/attributes/attributes[0]/code_length", 5)
        self.checkValue(parser, "/attributes_count", 3)
        self.checkValue(
            parser, "/attributes/attributes[2]/classes/classes[1]/inner_name_index", 83)

    def test_swf(self):
        parser = self.parse("claque-beignet.swf")
        self.checkDesc(parser, "rect", "Rectangle: 550x400")
        self.checkDisplay(parser, "frame_rate", "24.0")
        self.checkDesc(parser, "bkgd_color[0]/color", "RGB color: #CC9933")
        self.checkDisplay(parser, "def_sound[0]/rate", "11.0 kHz")
        self.checkValue(parser, "def_sound[0]/len", 1661)
        self.checkValue(parser, "sound_hdr2[0]/sound_is_16bit", False)
        self.checkValue(parser, "export[0]/export[0]/name", "C bras")

    def test_flv(self):
        parser = self.parse("breakdance.flv")
        self.checkDisplay(parser, "/audio[0]/codec", "MP3")
        self.checkValue(parser, "/audio[2]/timestamp", 52)
        self.checkDisplay(parser, "/video[0]/codec", "Sorensen H.263")
        self.checkValue(
            parser, "/metadata/entry[1]/item[8]/attr[1]/item[4]/value/exponent", 20)

    def test_tcpdump(self):
        parser = self.parse("arp_dns_ping_dns.tcpdump")
        self.checkValue(parser, "/packet[5]/ipv4/ttl", 120)
        self.checkDisplay(parser, "/packet[3]/ts_epoch", "2006-11-23 23:13:19")
        self.checkValue(parser, "/packet[3]/ipv4/src", "212.27.54.252")
        self.checkDisplay(parser, "/packet[7]/udp/src", "DNS")

    def test_ext2(self):
        parser = self.parse("my60k.ext2")
        self.checkDisplay(parser, "/superblock/last_check",
                          '2006-12-04 22:56:37')
        self.checkDisplay(parser, "/superblock/creator_os", "Linux")
        self.checkDisplay(parser, "/superblock/errors", "Continue")
        self.checkValue(parser, "/group_desc/group[0]/block_bitmap", 3)
        self.checkValue(parser, "/group_desc/group[0]/free_blocks_count", 44)
        self.checkValue(parser, "/group[0]/block_bitmap/item[9]", False)
        self.checkDisplay(
            parser, "/group[0]/inode_table/inode[1]/mode/file_type", "Directory")
        self.checkDesc(parser, "/group[0]/inode_table/inode[10]",
                       "Inode 11: Directory, size=1024 bytes, mode=drwxr-xr-x")
        self.checkValue(parser, "/group[0]/inode_table/inode[11]/size", 1816)
        self.checkDisplay(
            parser, "/group[0]/inode_table/inode[11]/ctime", '2006-12-04 23:22:00')
        self.checkValue(
            parser, "/superblock/feature_compat/dir_prealloc", False)
        self.checkValue(
            parser, "/superblock/feature_compat/imagic_inodes", False)
        self.checkValue(
            parser, "/superblock/feature_compat/has_journal", False)
        self.checkValue(parser, "/superblock/feature_compat/ext_attr", False)
        self.checkValue(parser, "/superblock/feature_compat/resize_inode", True)
        self.checkValue(parser, "/superblock/feature_compat/dir_index", True)
        self.checkValue(
            parser, "/superblock/feature_incompat/compression", False)
        self.checkValue(parser, "/superblock/feature_incompat/filetype", True)
        self.checkValue(parser, "/superblock/feature_incompat/recover", False)
        self.checkValue(
            parser, "/superblock/feature_incompat/journal_dev", False)
        self.checkValue(parser, "/superblock/feature_incompat/meta_bg", False)
        self.checkValue(
            parser, "/superblock/feature_ro_compat/sparse_super", True)
        self.checkValue(
            parser, "/superblock/feature_ro_compat/large_file", False)
        self.checkValue(
            parser, "/superblock/feature_ro_compat/btree_dir", False)
        self.checkValue(parser, "/superblock/feature_incompat/filetype", True)
        self.checkValue(parser, "/superblock/feature_incompat/recover", False)
        self.checkDisplay(parser, "/superblock/def_hash_version", "TEA")
        self.checkValue(
            parser, "/group[0]/inode_table/inode[0]/flags/secrm", False)
        self.checkDisplay(parser, "/superblock/rev_level",
                          "V2 format w/ dynamic inode sizes")

    def test_ext2_default_mount_opts(self):
        parser = self.parse("default_mount_opts.ext2")
        self.checkValue(parser, "/superblock/default_mount_opts/debug", False)
        self.checkValue(
            parser, "/superblock/default_mount_opts/bsdgroups", False)
        self.checkValue(
            parser, "/superblock/default_mount_opts/xattr_user", True)
        self.checkValue(parser, "/superblock/default_mount_opts/acl", True)
        self.checkValue(parser, "/superblock/default_mount_opts/uid16", False)
        self.checkDisplay(parser, "/superblock/default_mount_opts/jmode", "none")

    def test_ext2_variety_inode_types(self):
        parser = self.parse("types.ext2")
        self.checkDesc(parser, "/group[0]/inode_table/inode[12]",
                       "Inode 13: Symbolic link (-> XYZ), size=3 bytes, mode=lrwxrwxrwx")

    def test_bmp2(self):
        parser = self.parse("article01.bmp")
        self.checkDisplay(parser, "/header/red_mask", '0x00ff0000')
        self.checkDisplay(parser, "/header/color_space",
                          "Business (Saturation)")
        self.checkValue(parser, "/pixels/line[94]/pixel[11]", 15265520)

    def test_reiserfs3(self):
        parser = self.parse("reiserfs_v3_332k.bin")
        self.checkValue(parser, "/superblock/root_block", 645)
        self.checkDisplay(parser, "/superblock/hash_function", "R5_HASH")
        self.checkValue(parser, "/superblock/tree_height", 3)

    def test_pcx(self):
        parser = self.parse("lara_croft.pcx")
        self.checkDesc(parser, "/palette_4bits/color[8]", "RGB color: #100000")
        self.checkDesc(parser, "/palette_8bits/color[0]", "RGB color: Black")
        self.checkValue(parser, "/compression", 1)
        self.checkValue(parser, "/horiz_dpi", 500)

    def test_linux_swap(self):
        parser = self.parse("linux_swap_9pages")
        self.checkValue(parser, "/version", 1)
        self.checkValue(parser, "/last_page", 9)
        self.checkValue(parser, "/magic", "SWAPSPACE2")

    def test_wmf(self):
        parser = self.parse("pikachu.wmf")
        self.checkValue(parser, "/max_record_size", 510)
        self.checkValue(parser, "/func[2]/y", 10094)
        self.checkDisplay(parser, "/func[4]/brush_style", "Solid")
        self.checkValue(parser, "/func[10]/object_id", 2)

    def test_wmf2(self):
        parser = self.parse("globe.wmf")
        self.checkValue(parser, "/file_size", 3923)
        self.checkValue(parser, "/func[1]/x", 9989)
        self.checkDisplay(parser, "/func[4]/operation", "Copy pen (P)")
        self.checkDisplay(parser, "/func[9]/brush_style", "Null")

    def test_midi(self):
        parser = self.parse("indiana.mid")
        self.checkDesc(parser, "/header",
                       "Multiple tracks, synchronous; 3 tracks")
        self.checkDisplay(
            parser, "/track[0]/command[1]/microsec_quarter", "300.00 ms")
        self.checkDisplay(parser, "/track[1]/command[6]/note", "A (octave 5)")
        self.checkValue(parser, "/track[1]/command[8]/time", 408)
        self.checkValue(parser, "/track[1]/command[8]/velocity", 80)

    def test_wmf_emf(self):
        parser = self.parse("grasslogo_vector.emf")
        self.checkValue(parser, "/func[4]/y", 297)
        self.checkDesc(parser, "/func[15]", "Begin path")
        self.checkDesc(parser, "/func[40]/color",
                       "RGB color: #008F00 (opacity: 0%)")
        self.checkValue(parser, "/emf_header/maj_ver", 1)
        self.checkValue(parser, "/emf_header/width_px", 1024)
        self.checkValue(parser, "/emf_header/width_mm", 270)
        self.checkValue(parser, "/emf_header/description",
                        "Adobe Illustrator EMF 8.0")

    def test_gif(self):
        parser = self.parse("india_map.gif")
        self.checkValue(parser, "/screen/global_map", True)
        self.checkDesc(parser, "/color_map/color[0]", "RGB color: Black")
        self.checkValue(parser, "/image[0]/height", 794)

    def test_exe_msdos(self):
        parser = self.parse("cercle.exe")
        self.checkValue(parser, "/msdos/reloc_offset", 28)
        self.checkDisplay(parser, "/msdos/init_cs_ip", "0x00000315")

    def test_exe_pe(self):
        parser = self.parse("eula.exe")
        self.checkDisplay(parser, "/pe_header/cpu", "Intel 80386")
        self.checkDisplay(parser, "/pe_opt_header/subsystem", "Windows GUI")
        self.checkDisplay(parser, "/pe_opt_header/import/rva", "0x00008314")
        self.checkDisplay(parser, "/section_hdr[1]/mem_size", "4632 bytes")
        self.checkValue(parser, "/section_hdr[1]/is_readable", True)
        self.checkValue(parser, "/section_hdr[1]/is_executable", False)
        self.checkValue(
            parser, "/section_rsrc/version_info/node[0]/node[1]/node[0]/node[0]/value", "Dell Inc")
        self.checkDesc(
            parser, "/section_rsrc/icon[0]/bmp_header", "Bitmap info header: 16x32 pixels, 4 bits/pixel")
        self.checkDesc(
            parser, "/section_rsrc/icon[1]", "Resource #296 content: type=3")

    def test_laf(self):
        parser = self.parse("ocr10.laf")
        self.checkValue(parser, "/chars/char[3]/data_offset", 201)
        self.checkValue(parser, "/chars/char[8]/width_pixels", 10)

    def test_laf2(self):
        parser = self.parse("kino14s.laf")
        self.checkValue(parser, "/max_char_width", 26)
        self.checkValue(parser, "/char_codes/char[5]", 5)
        self.checkValue(parser, "/chars/char[1]/data_offset", 1)
        self.checkValue(parser, "/chars/char[1]/height_pixels", 13)
        self.checkValue(
            parser, "/char_data/char_bitmap[3]/line[1]/pixel[0]", 128)

    def test_bz2(self):
        parser = self.parse("free-software-song.midi.bz2")
        self.checkDisplay(parser, "/blocksize", "'9'")
        self.checkDisplay(parser, "/file/crc32", "0x8c3c1b7b")

    def test_elf_program_32lsb(self):
        parser = self.parse("ping_20020927-3ubuntu2")
        self.checkDisplay(parser, "/header/class", "32 bits")
        self.checkDisplay(parser, "/header/endian", "Little endian")
        self.checkDisplay(parser, "/header/type", "Executable file")
        self.checkDisplay(parser, "/header/machine", "Intel 80386")
        self.checkValue(parser, "/header/phentsize", 32)
        self.checkDisplay(parser, "/prg_header[1]/type", "Program interpreter")
        self.checkValue(
            parser, "/section_header[0]/flags/is_writable", False)
        self.checkValue(
            parser, "/section_header[0]/flags/is_alloc", False)
        self.checkValue(
            parser, "/section_header[0]/flags/is_exec", False)
        self.checkValue(
            parser, "/section_header[0]/flags/is_tls", False)
        self.checkValue(
            parser, "/section_header[1]/flags/is_writable", False)
        self.checkValue(
            parser, "/section_header[1]/flags/is_alloc", True)
        self.checkValue(
            parser, "/section_header[1]/flags/is_exec", False)
        self.checkValue(
            parser, "/section_header[1]/flags/is_tls", False)
        self.checkValue(
            parser, "/section_header[10]/flags/is_writable", False)
        self.checkValue(
            parser, "/section_header[10]/flags/is_alloc", True)
        self.checkValue(
            parser, "/section_header[10]/flags/is_exec", True)
        self.checkValue(
            parser, "/section_header[10]/flags/is_tls", False)
        self.checkValue(
            parser, "/section_header[17]/flags/is_writable", True)
        self.checkValue(
            parser, "/section_header[17]/flags/is_alloc", True)
        self.checkValue(
            parser, "/section_header[17]/flags/is_exec", False)
        self.checkValue(
            parser, "/section_header[17]/flags/is_tls", False)

    def test_elf_program_64lsb(self):
        parser = self.parse("get-versions.64bit.little.elf")
        self.checkDisplay(parser, "/header/endian", "Little endian")
        self.checkDisplay(parser, "/header/type", "Executable file")
        self.checkDisplay(parser, "/header/machine", "Advanced Micro Devices x86-64")
        self.checkValue(parser, "/header/phentsize", 56)
        self.checkDisplay(parser, "/prg_header[1]/type", "Program interpreter")
        self.checkValue(
            parser, "/section_header[0]/flags/is_writable", False)
        self.checkValue(
            parser, "/section_header[0]/flags/is_alloc", False)
        self.checkValue(
            parser, "/section_header[0]/flags/is_exec", False)
        self.checkValue(
            parser, "/section_header[0]/flags/is_tls", False)
        self.checkValue(
            parser, "/section_header[1]/flags/is_writable", False)
        self.checkValue(
            parser, "/section_header[1]/flags/is_alloc", True)
        self.checkValue(
            parser, "/section_header[1]/flags/is_exec", False)
        self.checkValue(
            parser, "/section_header[1]/flags/is_tls", False)
        self.checkValue(
            parser, "/section_header[10]/flags/is_writable", False)
        self.checkValue(
            parser, "/section_header[10]/flags/is_alloc", True)
        self.checkValue(
            parser, "/section_header[10]/flags/is_exec", True)
        self.checkValue(
            parser, "/section_header[10]/flags/is_tls", False)
        self.checkValue(
            parser, "/section_header[18]/flags/is_writable", True)
        self.checkValue(
            parser, "/section_header[18]/flags/is_alloc", True)
        self.checkValue(
            parser, "/section_header[18]/flags/is_exec", False)
        self.checkValue(
            parser, "/section_header[18]/flags/is_tls", False)

    def test_elf_program_32msb(self):
        parser = self.parse("mev.32bit.big.elf")
        self.checkDisplay(parser, "/header/class", "32 bits")
        self.checkDisplay(parser, "/header/endian", "Big endian")
        self.checkDisplay(parser, "/header/type", "Shared object file")
        self.checkDisplay(parser, "/header/machine", "MIPS I Architecture")
        self.checkValue(parser, "/header/phentsize", 32)
        self.checkDisplay(parser, "/prg_header[1]/type", "Program interpreter")
        self.checkValue(
            parser, "/section_header[0]/flags/is_writable", False)
        self.checkValue(
            parser, "/section_header[0]/flags/is_alloc", False)
        self.checkValue(
            parser, "/section_header[0]/flags/is_exec", False)
        self.checkValue(
            parser, "/section_header[0]/flags/is_tls", False)
        self.checkValue(
            parser, "/section_header[1]/flags/is_writable", False)
        self.checkValue(
            parser, "/section_header[1]/flags/is_alloc", True)
        self.checkValue(
            parser, "/section_header[1]/flags/is_exec", False)
        self.checkValue(
            parser, "/section_header[1]/flags/is_tls", False)
        self.checkValue(
            parser, "/section_header[13]/flags/is_writable", False)
        self.checkValue(
            parser, "/section_header[13]/flags/is_alloc", True)
        self.checkValue(
            parser, "/section_header[13]/flags/is_exec", True)
        self.checkValue(
            parser, "/section_header[13]/flags/is_tls", False)
        self.checkValue(
            parser, "/section_header[19]/flags/is_writable", True)
        self.checkValue(
            parser, "/section_header[19]/flags/is_alloc", True)
        self.checkValue(
            parser, "/section_header[19]/flags/is_exec", False)
        self.checkValue(
            parser, "/section_header[19]/flags/is_tls", False)

    def test_elf_program_64msb(self):
        parser = self.parse("mev.64bit.big.elf")
        self.checkDisplay(parser, "/header/class", "64 bits")
        self.checkDisplay(parser, "/header/endian", "Big endian")
        self.checkDisplay(parser, "/header/type", "Shared object file")
        self.checkDisplay(parser, "/header/machine", "IBM S390")
        self.checkValue(parser, "/header/phentsize", 56)
        self.checkDisplay(parser, "/prg_header[1]/type", "Program interpreter")
        self.checkValue(
            parser, "/section_header[0]/flags/is_writable", False)
        self.checkValue(
            parser, "/section_header[0]/flags/is_alloc", False)
        self.checkValue(
            parser, "/section_header[0]/flags/is_exec", False)
        self.checkValue(
            parser, "/section_header[0]/flags/is_tls", False)
        self.checkValue(
            parser, "/section_header[1]/flags/is_writable", False)
        self.checkValue(
            parser, "/section_header[1]/flags/is_alloc", True)
        self.checkValue(
            parser, "/section_header[1]/flags/is_exec", False)
        self.checkValue(
            parser, "/section_header[1]/flags/is_tls", False)
        self.checkValue(
            parser, "/section_header[13]/flags/is_writable", False)
        self.checkValue(
            parser, "/section_header[13]/flags/is_alloc", True)
        self.checkValue(
            parser, "/section_header[13]/flags/is_exec", True)
        self.checkValue(
            parser, "/section_header[13]/flags/is_tls", False)
        self.checkValue(
            parser, "/section_header[19]/flags/is_writable", True)
        self.checkValue(
            parser, "/section_header[19]/flags/is_alloc", True)
        self.checkValue(
            parser, "/section_header[19]/flags/is_exec", False)
        self.checkValue(
            parser, "/section_header[19]/flags/is_tls", False)
        self.checkValue(
            parser, "/section_header[10]/flags/is_info_link", True)

    def test_cab(self):
        parser = self.parse("georgia.cab")
        self.checkDisplay(parser, "/minor_version", "3")
        self.checkDisplay(parser, "/major_version", "1")
        self.checkDesc(parser, "/file[0]", 'File "fontinst.inf" (64 bytes)')
        self.checkValue(parser, "/file[1]/filename", "Georgiaz.TTF")

    def test_torrent(self):
        parser = self.parse("debian-31r4-i386-binary-1.iso.torrent")
        self.checkValue(parser, "/root/announce",
                        "http://bttracker.acc.umu.se:6969/announce")
        self.checkValue(parser, "/root/comment",
                        '"Debian CD from cdimage.debian.org"')
        self.checkDisplay(parser, "/root/creation_date", '2006-11-16 21:44:37')
        self.checkDisplay(parser, "/root/info/value/length", "638.7 MB")
        self.checkDisplay(parser, "/root/info/value/piece_length", "512.0 KB")

    def test_fat16(self):
        parser = self.parse("dell8.fat16")
        self.checkValue(parser, "/boot/oem_name", "Dell 8.0")
        self.checkDisplay(parser, "/boot/serial", "0x07d6090d")
        self.checkValue(parser, "/boot/label", "DellUtility")
        self.checkValue(parser, "/boot/fs_type", "FAT16")
        self.checkValue(parser, "/fat[1]/entry[2]", 3)
        self.checkDisplay(parser, "/fat[0]/entry[4008]", "free cluster")
        self.checkDesc(parser, "/root[0]/entry[0]",
                       "Long filename part: 'command.com' [65]")
        self.checkDesc(parser, "/root[0]/entry[1]", "File: 'COMMAND.COM'")
        self.checkValue(parser, "/root[0]/entry[2]/hidden", True)
        self.checkDesc(
            parser, "/root[0]/entry[2]/create", "2006-09-13 15:01:16")
        self.checkDesc(parser, "/root[0]/entry[2]/access", "2006-09-13")
        self.checkDesc(
            parser, "/root[0]/entry[2]/modify", "2005-07-26 00:48:26")
        self.checkValue(parser, "/root[0]/entry[2]/size", 29690)

    def test_xm(self):
        parser = self.parse("dontyou.xm")
        self.checkValue(parser, "/header/title", "Dont you... voguemix")
        self.checkValue(parser, "/header/bpm", 128)
        self.checkValue(parser, "/pattern[0]/data_size", 708)
        self.checkDisplay(
            parser, "/pattern[0]/row[0]/note[0]/note", "F (octave 5)")
        self.checkValue(
            parser, "/pattern[0]/row[0]/note[0]/effect_parameter", 0x0A)
        self.checkDisplay(
            parser, "/pattern[0]/row[0]/note[1]/note", "A (octave 5)")
        self.checkValue(
            parser, "/pattern[0]/row[0]/note[1]/effect_parameter", 0x80)
        self.checkValue(parser, "/pattern[20]/data_size", 1129)
        self.checkDisplay(
            parser, "/pattern[20]/row[0]/note[0]/note", "C# (octave 5)")
        self.checkValue(parser, "/pattern[20]/row[0]/note[0]/instrument", 5)
        self.checkValue(parser, "/instrument[3]/name", "Vogue of Triton")
        self.checkValue(
            parser, "/instrument[4]/second_header/panning_points", 6)
        self.checkValue(parser, "/instrument[20]/name", "808-hi5.smp")

    def test_s3m(self):
        parser = self.parse("satellite_one.s3m")
        self.checkValue(parser, "/header/title", "Satellite one.")
        self.checkValue(parser, "/instrument[0]/sample_offset", 27024)
        self.checkValue(parser, "/instrument[0]/c4_speed", 8363)
        self.checkValue(parser, "/instrument[0]/name", "By Purple Motion of")
        self.checkValue(parser, "/instrument[1]/name", "Future Crew - 1993")
        self.checkValue(parser, "/pattern[0]/row[0]/note[0]/volume", 54)

    def test_ptm(self):
        parser = self.parse("anti-arpeggio_tune.ptm")
        self.checkValue(parser, "/header/title", "Anti-arpeggio tune!-VV/AcMe")
        self.checkValue(parser, "/instrument[0]/sample_offset", 30928)
        self.checkValue(parser, "/instrument[0]/c4_speed", 8363)
        self.checkValue(parser, "/instrument[0]/name", "Yep guess wat....")
        self.checkValue(
            parser, "/instrument[1]/name", "Mag ik even mijn ongenoegen")
        self.checkValue(parser, "/instrument[1]/gus_loop_flags", 0)
        self.checkValue(parser, "/pattern[0]/row[0]/note[0]/effect", 15)

    def test_lnk(self):
        parser = self.parse("vim.lnk")
        self.checkValue(parser, "/creation_time",
                        datetime(2006, 5, 7, 14, 18, 32))
        self.checkValue(parser, "/target_filesize", 1363968)

    def test_chm(self):
        parser = self.parse("7zip.chm")
        self.checkValue(parser, "/itsf/version", 3)
        self.checkDisplay(parser, "/itsf/lang_id", "Russian")
        self.checkValue(parser, "/itsf/dir_uuid/time",
                        datetime(1997, 1, 31, 20, 42, 14, 890626))
        self.checkDisplay(parser, "/itsf/stream_uuid/variant",
                          "Microsoft Corporation")
        self.checkDisplay(parser, "/itsf/stream_uuid/mac",
                          'INTEL CORPORATION - HF1-06 [22:e6:ec]')
        self.checkDisplay(parser, "/file_size/file_size", "75.6 KB")
        self.checkDisplay(parser, "/dir/itsp/lang_id", "English United States")
        self.checkValue(parser, "/dir/pmgl[0]/entry[1]/name", "/#IDXHDR")

    def test_blp(self):
        parser = self.parse("swat.blp")
        self.checkValue(parser, "flags", 8)
        self.checkValue(parser, "height", 256)
        self.checkValue(parser, "width", 256)
        self.checkValue(parser, "jpeg_header_len", 10)

    def test_nds(self):
        parser = self.parse("nitrodir.nds")
        self.checkValue(parser, "/header/game_title", '.')
        self.checkValue(parser, "/header/header_crc16", 29398)
        self.checkValue(parser, "/banner/icon_data/tile[3,3]/pixel[7,6]", 5)
        self.checkValue(parser, "/banner/palette_color[13]/blue", 28)
        self.checkValue(
            parser, "/filename_table/directory[3]/entry[1]/name", "subsubdir1")
        self.checkValue(
            parser, "/filename_table/directory[3]/entry[1]/dir_id", 61444)
        self.checkValue(
            parser, "/filename_table/directory[4]/entry[0]/name", "file2.txt")
        self.checkValue(
            parser, "/filename_table/directory[4]/entry[0]/is_directory", False)
        self.checkValue(parser, "/file[1]", b"Hello from file2.txt\n\n")

    def test_prs_pak(self):
        parser = self.parse("paktest.pak")
        self.checkValue(
            parser, "/file[0]/filename", "hachoir/png_331x90x8_truncated.png")
        self.checkValue(parser, "/file[0]/size", 100)
        self.checkValue(
            parser, "/file[1]/filename", "hachoir/small_text.tar")
        self.checkValue(parser, "/file[1]/size", 10240)

    def test_7zip(self):
        parser = self.parse("archive.7z")
        self.checkValue(parser, "/signature/major_ver", 0)
        self.checkValue(parser, "/signature/minor_ver", 4)
        self.checkValue(
            parser, "/next_hdr/encoded_hdr/pack_info/pack_pos", 96634)
        self.checkDisplay(
            parser, "/next_hdr/encoded_hdr/pack_info/size_marker", 'kSize')
        self.checkValue(
            parser, "/next_hdr/encoded_hdr/unpack_info/num_folders", 1)

    def test_java_serialized(self):
        parser = self.parse("weka.model")
        self.checkDisplay(
            parser, "/object[0]/classDesc", "weka.classifiers.functions.SMO")
        self.checkDisplay(
            parser, "/object[1]/classDesc", "-> weka.core.Instances")
        self.checkDesc(
            parser, "/object[1]/value/field[2]/value/field[0]",
            "java.util.ArrayList.size")
        self.checkValue(
            parser, "/object[1]/value/field[2]/value/field[0]", 11)

    def test_mpeg_ts(self):
        parser = self.parse("sample.ts")
        self.checkValue(
            parser, "/packet[0]/has_error", False)
        self.checkValue(
            parser, "/packet[0]/has_payload", True)
        self.checkValue(
            parser, "/packet[2]/has_adaptation", True)
        self.checkValue(
            parser, "/packet[2]/adaptation_field/pcr_base", 44)
        self.checkValue(
            parser, "/packet[78]/payload_unit_start", True)

    def test_m2ts(self):
        parser = self.parse("Panasonic_AG_HMC_151.MTS")
        self.checkValue(
            parser, "/packet[0]/has_error", False)
        self.checkValue(
            parser, "/packet[0]/has_payload", True)
        self.checkValue(
            parser, "/packet[3]/has_adaptation", True)
        self.checkValue(
            parser, "/packet[3]/adaptation_field/pcr_base", 153)

    def test_macho_x86_ppc(self):
        parser = self.parse("macos_10.5.macho")
        self.checkDisplay(
            parser, "/file[0]/header/cputype", "PowerPC")
        self.checkDisplay(
            parser, "/file[1]/header/cputype", "i386")
        self.checkValue(
            parser, "/file[0]/load_command[1]/data/segname", "__TEXT")
        self.checkValue(
            parser, "/file[0]/load_command[1]/data/section[1]/sectname", "__symbol_stub1")
        self.checkValue(
            parser, "/file[1]/load_command[1]/data/segname", "__TEXT")
        self.checkValue(
            parser, "/file[1]/load_command[1]/data/section[1]/sectname", "__cstring")

    def test_macho_x86_64(self):
        parser = self.parse("macos_10.12.macho")
        self.checkDisplay(
            parser, "/file[0]/header/cputype", "i386")
        self.checkDisplay(
            parser, "/file[1]/header/cputype", "x86_64")
        self.checkValue(
            parser, "/file[0]/load_command[1]/data/segname", "__TEXT")
        self.checkValue(
            parser, "/file[0]/load_command[1]/data/section[1]/sectname", "__symbol_stub")
        self.checkValue(
            parser, "/file[1]/load_command[1]/data/segname", "__TEXT")
        self.checkValue(
            parser, "/file[1]/load_command[1]/data/section[1]/sectname", "__stubs")

    def test_cr2(self):
        parser = self.parse("canon.raw.cr2")
        self.checkValue(parser, "version", 42)
        self.checkValue(parser, "cr_identifier", "CR")
        self.checkValue(parser, "ifd[0]/entry[0]/tag", 0x0100)
        self.checkValue(parser, "ifd[0]/value[4]", 'Canon')
        self.checkValue(parser, "ifd[0]/value[5]", 'Canon EOS REBEL T5i')

    def test_tga(self):
        parser = self.parse("32bpp.tga")
        self.checkValue(parser, "/width", 800)
        self.checkValue(parser, "/height", 600)
        self.checkValue(parser, "/bpp", 32)
        self.checkDisplay(parser, "/codec", "True-color RLE")


class TestParserRandomStream(unittest.TestCase):

    def test_random_stream(self, tests=(1, 8)):
        a = array('L')
        parser_list = HachoirParserList()
        n = max(tests) * max(parser.getParserTags()
                             ["min_size"] for parser in parser_list)
        k = 8 * a.itemsize
        for i in range((n - 1) // k + 1):
            a.append(random.getrandbits(k))
        a = StringInputStream(a.tobytes(), source="<random data>")
        for parser in parser_list:
            size = parser.getParserTags()["min_size"]
            for test in tests:
                a._size = a._current_size = size * test
                try:
                    parser(a, validate=True)
                    error("[%s] Parser didn't reject random data" %
                          parser.__name__)
                except ValidateError:
                    continue


if __name__ == "__main__":
    setup_tests()
    unittest.main()



#!/usr/bin/env python3
from zlib import decompress, error as zlib_error
from sys import argv, stderr, exit
from time import time

MIN_SIZE = 2


def canDeflate(compressed_data):
    try:
        data = decompress(compressed_data)
        return True
    except zlib_error:
        return False


def findDeflateBlocks(data):
    next_msg = time() + 1.0
    max_index = len(data) - MIN_SIZE - 1
    for index in range(max_index + 1):
        if next_msg < time():
            next_msg = time() + 1.0
            print("Progress: %.1f%% (offset %s/%s)" % (
                index * 100.0 / max_index, index, max_index))
        if canDeflate(data[index:]):
            yield index


def guessDeflateSize(data, offset):
    size = len(data) - offset
    while size:
        if canDeflate(data[offset:offset + size]):
            yield size
        size -= 1


def main():
    if len(argv) != 2:
        print("usage: %s filename" % argv[0], file=stderr)
        exit(1)
    data = open(argv[1], 'rb').read()
    offsets = []
    for offset in findDeflateBlocks(data):
        print("Offset %s" % offset)
        offsets.append(offset)
    if offsets:
        for offset in offsets:
            for size in guessDeflateSize(data, offset):
                if size == (len(data) - offset):
                    size = "%s (until the end)" % size
                print("Offset %s -- size %s" % (offset, size))
    else:
        print("No deflate block found", file=stderr)
    exit(0)

if __name__ == "__main__":
    main()



from random import randint, choice as random_choice
from array import array

MAX_MIX = 20
MIN_MIX = -MAX_MIX
MIN_COUNT = 15
MAX_COUNT = 2500
MAX_INC = 32
MIN_INC = -MAX_INC

SPECIAL_VALUES_NOENDIAN = (
    "\x00",
    "\x00\x00",
    "\x7f",
    "\x7f\xff",
    "\x7f\xff\xff\xff",
    "\x80",
    "\x80\x00",
    "\x80\x00\x00\x00",
    "\xfe",
    "\xfe\xff",
    "\xfe\xff\xff\xff",
    "\xff",
    "\xff\xff",
    "\xff\xff\xff\xff",
)

SPECIAL_VALUES = []
for item in SPECIAL_VALUES_NOENDIAN:
    SPECIAL_VALUES.append(item)
    itemb = item[::-1]
    if item != itemb:
        SPECIAL_VALUES.append(itemb)


def mangle_replace(data, offset):
    data[offset] = randint(0, 255)


def mangle_increment(data, offset):
    value = data[offset] + randint(MIN_INC, MAX_INC)
    data[offset] = max(min(value, 255), 0)


def mangle_bit(data, offset):
    bit = randint(0, 7)
    if randint(0, 1) == 1:
        value = data[offset] | (1 << bit)
    else:
        value = data[offset] & (~(1 << bit) & 0xFF)
    data[offset] = value


def mangle_special_value(data, offset):
    tlen = len(data) - offset
    text = random_choice(SPECIAL_VALUES)[:tlen]
    data[offset:offset + len(text)] = array("B", text)


def mangle_mix(data, ofs1):
    ofs2 = ofs1 + randint(MIN_MIX, MAX_MIX)
    ofs2 = max(min(ofs2, len(data) - 1), 0)
    data[ofs1], data[ofs2] = data[ofs2], data[ofs1]


MANGLE_OPERATIONS = (
    mangle_replace,
    mangle_increment,
    mangle_bit,
    mangle_special_value,
    mangle_mix,
)


def mangle(data, percent, min_count=MIN_COUNT, max_count=MAX_COUNT):
    """
    Mangle data: add few random bytes in input byte array.

    This function is based on an idea of Ilja van Sprundel (file mangle.c).
    """
    hsize = len(data) - 1
    max_percent = max(min(percent, 1.0), 0.0001)
    count = int(float(len(data)) * max_percent)
    count = max(count, min_count)
    count = min(count, max_count)
    count = randint(1, count)
    for index in range(count):
        operation = random_choice(MANGLE_OPERATIONS)
        offset = randint(0, hsize)
        operation(data, offset)
    return count



#!/usr/bin/env python3
import doctest
import hachoir.core.i18n   # noqa: import it because it does change the locale
from hachoir.test import setup_tests
import os
import unittest

ROOT = os.path.normpath(os.path.join(os.path.dirname(__file__), '..'))


def importModule(name):
    mod = __import__(name)
    components = name.split('.')
    for comp in components[1:]:
        mod = getattr(mod, comp)
    return mod


class TestDoc(unittest.TestCase):
    verbose = False

    def check_doc(self, filename, subdir=None, name=None):
        if self.verbose:
            print("--- %s: Run tests" % filename)
        if not subdir:
            fullpath = os.path.join('..', 'doc', filename)
        else:
            fullpath = os.path.join(subdir, filename)
        failure, nb_test = doctest.testfile(
            fullpath, optionflags=doctest.ELLIPSIS, name=name)
        if failure:
            self.fail("error")
        if self.verbose:
            print("--- %s: End of tests" % filename)

    def check_module(self, name):
        if self.verbose:
            print("--- Test module %s" % name)
        module = importModule(name)
        failure, nb_test = doctest.testmod(module)
        if failure:
            self.fail("error")
        if self.verbose:
            print("--- End of test")

    def test_doc_directory(self):
        self.check_doc('developer.rst')
        self.check_doc('internals.rst')
        self.check_doc('regex.rst')

    def test_tests_directory(self):
        self.check_doc('regex_regression.rst', subdir='.')

    def test_hachoir_core(self):
        self.check_module("hachoir.core.bits")
        self.check_module("hachoir.core.dict")
        self.check_module("hachoir.core.i18n")
        self.check_module("hachoir.core.text_handler")
        self.check_module("hachoir.core.tools")

    def test_hachoir_metadata(self):
        self.check_module("hachoir.metadata.metadata")
        self.check_module("hachoir.metadata.setter")

    def test_hachoir_regex(self):
        self.check_module("hachoir.regex.parser")
        self.check_module("hachoir.regex.regex")
        self.check_module("hachoir.regex.pattern")


if __name__ == "__main__":
    setup_tests()
    unittest.main()



#!/usr/bin/env python3
#
# Prepare a release:
#
#  - check version: hachoir/version.py and doc/conf.py
#  - set the release date: edit doc/changelog.rst
#  - run: git commit -a
#  - Remove untracked files/dirs: git clean -fdx
#  - run tests: tox
#  - run: git push
#  - check Travis CI status:
#    https://travis-ci.org/vstinner/hachoir
#  - run: git tag x.y.z
#  - Remove untracked files/dirs: git clean -fdx
#  - run: python3 setup.py sdist bdist_wheel
#
# Release a new version:
#
#  - git push --tags
#  - twine upload dist/*
#
# After the release:
#
#  - set version to N+1: hachoir/version.py and doc/conf.py

ENTRY_POINTS = {
    'console_scripts': [
        "hachoir-grep = hachoir.grep:main",
        "hachoir-metadata = hachoir.metadata.main:main",
        "hachoir-strip = hachoir.strip:main",
        "hachoir-urwid = hachoir.urwid:main"
    ],
    'gui_scripts': [
        "hachoir-wx = hachoir.wx.main:main"
    ]
}
# FIXME: hachoir-subfile is currently broken
# "hachoir-subfile",

CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Environment :: Console :: Curses',
    'Environment :: Plugins',
    'Intended Audience :: Developers',
    'Intended Audience :: Education',
    'License :: OSI Approved :: GNU General Public License (GPL)',
    'Natural Language :: English',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 3',
    'Topic :: Multimedia',
    'Topic :: Scientific/Engineering :: Information Analysis',
    'Topic :: Software Development :: Disassemblers',
    'Topic :: Software Development :: Interpreters',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: System :: Filesystems',
    'Topic :: Text Processing',
    'Topic :: Utilities',
]


def main():
    from setuptools import setup
    from setuptools import find_packages

    import hachoir

    readme = open('README.rst')
    long_description = readme.read()
    readme.close()

    install_options = {
        "name": "hachoir",
        "version": hachoir.__version__,
        "url": 'http://hachoir.readthedocs.io/',
        "author": "Hachoir team (see AUTHORS file)",
        "description": "Package of Hachoir parsers used to open binary files",
        "long_description": long_description,
        "classifiers": CLASSIFIERS,
        "license": 'GNU GPL v2',
        "packages": find_packages(),
        "package_data": {"hachoir.wx.resource": ['hachoir_wx.xrc']},
        "entry_points": ENTRY_POINTS,
        "extras_require": {
            "urwid": [
                "urwid==1.3.1"
            ]
        },
        "zip_safe": True,
    }
    setup(**install_options)


if __name__ == "__main__":
    main()
